# cannot be used for CFD
import numpy


T = [
    "CGNSTree",
    None,
    [
        [
            "CGNSLibraryVersion",
            numpy.array([3.200000047683716], dtype="float32", order="C"),
            [],
            "CGNSLibraryVersion_t",
        ],
        [
            "{Base#1}",
            numpy.array([3, 3], dtype="int32", order="C"),
            [
                [
                    "Interior",
                    None,
                    [
                        [
                            "FamilyBC",
                            numpy.array(
                                ["B", "C", "F", "a", "r", "f", "i", "e", "l", "d"],
                                dtype="S",
                                order="C",
                            ),
                            [],
                            "FamilyBC_t",
                        ],
                    ],
                    "Family_t",
                ],
                [
                    "Exterior",
                    None,
                    [
                        [
                            "FamilyBC",
                            numpy.array(
                                ["B", "C", "W", "a", "l", "l"], dtype="S", order="C"
                            ),
                            [],
                            "FamilyBC_t",
                        ],
                    ],
                    "Family_t",
                ],
                [
                    "{Zone-A}",
                    numpy.array(
                        [[13, 12, 0], [61, 60, 0], [17, 16, 0]],
                        dtype="int64",
                        order="F",
                    ),
                    [
                        [
                            "ZoneType",
                            numpy.array(
                                ["S", "t", "r", "u", "c", "t", "u", "r", "e", "d"],
                                dtype="S",
                                order="C",
                            ),
                            [],
                            "ZoneType_t",
                        ],
                        [
                            "GridCoordinates",
                            None,
                            [
                                ["CoordinateX", None, [], "DataArray_t"],
                                ["CoordinateY", None, [], "DataArray_t"],
                                ["CoordinateZ", None, [], "DataArray_t"],
                            ],
                            "GridCoordinates_t",
                        ],
                        [
                            "ZoneBC",
                            None,
                            [
                                [
                                    "{BC-2}",
                                    numpy.array(
                                        [
                                            "F",
                                            "a",
                                            "m",
                                            "i",
                                            "l",
                                            "y",
                                            "S",
                                            "p",
                                            "e",
                                            "c",
                                            "i",
                                            "f",
                                            "i",
                                            "e",
                                            "d",
                                        ],
                                        dtype="S",
                                        order="C",
                                    ),
                                    [
                                        [
                                            "PointRange",
                                            numpy.array(
                                                [[1, 13], [1, 1], [1, 17]],
                                                dtype="int32",
                                                order="F",
                                            ),
                                            [],
                                            "IndexRange_t",
                                        ],
                                        [
                                            "FamilyName",
                                            numpy.array(
                                                [
                                                    "I",
                                                    "n",
                                                    "t",
                                                    "e",
                                                    "r",
                                                    "i",
                                                    "o",
                                                    "r",
                                                ],
                                                dtype="S",
                                                order="C",
                                            ),
                                            [],
                                            "FamilyName_t",
                                        ],
                                    ],
                                    "BC_t",
                                ],
                                [
                                    "{BC-1}",
                                    numpy.array(
                                        [
                                            "F",
                                            "a",
                                            "m",
                                            "i",
                                            "l",
                                            "y",
                                            "S",
                                            "p",
                                            "e",
                                            "c",
                                            "i",
                                            "f",
                                            "i",
                                            "e",
                                            "d",
                                        ],
                                        dtype="S",
                                        order="C",
                                    ),
                                    [
                                        [
                                            "PointRange",
                                            numpy.array(
                                                [[1, 13], [1, 61], [1, 1]],
                                                dtype="int32",
                                                order="F",
                                            ),
                                            [],
                                            "IndexRange_t",
                                        ],
                                        [
                                            "FamilyName",
                                            numpy.array(
                                                [
                                                    "E",
                                                    "x",
                                                    "t",
                                                    "e",
                                                    "r",
                                                    "i",
                                                    "o",
                                                    "r",
                                                ],
                                                dtype="S",
                                                order="C",
                                            ),
                                            [],
                                            "FamilyName_t",
                                        ],
                                    ],
                                    "BC_t",
                                ],
                                [
                                    "{BC-4}",
                                    numpy.array(
                                        [
                                            "F",
                                            "a",
                                            "m",
                                            "i",
                                            "l",
                                            "y",
                                            "S",
                                            "p",
                                            "e",
                                            "c",
                                            "i",
                                            "f",
                                            "i",
                                            "e",
                                            "d",
                                        ],
                                        dtype="S",
                                        order="C",
                                    ),
                                    [
                                        [
                                            "PointRange",
                                            numpy.array(
                                                [[1, 13], [61, 61], [1, 17]],
                                                dtype="int32",
                                                order="F",
                                            ),
                                            [],
                                            "IndexRange_t",
                                        ],
                                        [
                                            "FamilyName",
                                            numpy.array(
                                                [
                                                    "E",
                                                    "x",
                                                    "t",
                                                    "e",
                                                    "r",
                                                    "i",
                                                    "o",
                                                    "r",
                                                ],
                                                dtype="S",
                                                order="C",
                                            ),
                                            [],
                                            "FamilyName_t",
                                        ],
                                    ],
                                    "BC_t",
                                ],
                                [
                                    "{BC-6}",
                                    numpy.array(
                                        [
                                            "F",
                                            "a",
                                            "m",
                                            "i",
                                            "l",
                                            "y",
                                            "S",
                                            "p",
                                            "e",
                                            "c",
                                            "i",
                                            "f",
                                            "i",
                                            "e",
                                            "d",
                                        ],
                                        dtype="S",
                                        order="C",
                                    ),
                                    [
                                        [
                                            "PointRange",
                                            numpy.array(
                                                [[1, 13], [1, 61], [17, 17]],
                                                dtype="int32",
                                                order="F",
                                            ),
                                            [],
                                            "IndexRange_t",
                                        ],
                                        [
                                            "FamilyName",
                                            numpy.array(
                                                [
                                                    "E",
                                                    "x",
                                                    "t",
                                                    "e",
                                                    "r",
                                                    "i",
                                                    "o",
                                                    "r",
                                                ],
                                                dtype="S",
                                                order="C",
                                            ),
                                            [],
                                            "FamilyName_t",
                                        ],
                                    ],
                                    "BC_t",
                                ],
                            ],
                            "ZoneBC_t",
                        ],
                        [
                            "ZoneGridConnectivity",
                            None,
                            [
                                [
                                    "{CT-A-D}",
                                    numpy.array(
                                        ["{", "Z", "o", "n", "e", "-", "D", "}"],
                                        dtype="S",
                                        order="C",
                                    ),
                                    [
                                        [
                                            "GridConnectivityType",
                                            numpy.array(
                                                [
                                                    "A",
                                                    "b",
                                                    "u",
                                                    "t",
                                                    "t",
                                                    "i",
                                                    "n",
                                                    "g",
                                                ],
                                                dtype="S",
                                                order="C",
                                            ),
                                            [],
                                            "GridConnectivityType_t",
                                        ],
                                        [
                                            "ConnectivityKeyLocal",
                                            numpy.array(
                                                ["A", "_", "D"], dtype="S", order="C"
                                            ),
                                            [],
                                            "DataArray_t",
                                        ],
                                        [
                                            "ConnectivityKeyDonor",
                                            numpy.array(
                                                ["D", "_", "A"], dtype="S", order="C"
                                            ),
                                            [],
                                            "DataArray_t",
                                        ],
                                        [
                                            "PointRange",
                                            numpy.array(
                                                [[1, 1], [1, 61], [1, 17]],
                                                dtype="int32",
                                                order="F",
                                            ),
                                            [],
                                            "IndexRange_t",
                                        ],
                                        [
                                            ".Solver#Property",
                                            None,
                                            [
                                                [
                                                    "globborder",
                                                    numpy.array(
                                                        ["A", "_", "D"],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                                [
                                                    "globborderdonor",
                                                    numpy.array(
                                                        ["D", "_", "A"],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                                [
                                                    "jtype",
                                                    numpy.array(
                                                        [
                                                            "n",
                                                            "o",
                                                            "m",
                                                            "a",
                                                            "t",
                                                            "c",
                                                            "h",
                                                        ],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                                [
                                                    "nomatch_special",
                                                    numpy.array(
                                                        ["n", "o", "n", "e"],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                                [
                                                    "type",
                                                    numpy.array(
                                                        ["j", "o", "i", "n"],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                            ],
                                            "UserDefinedData_t",
                                        ],
                                    ],
                                    "GridConnectivity_t",
                                ],
                                [
                                    "{CT-A-B}",
                                    numpy.array(
                                        ["{", "Z", "o", "n", "e", "-", "B", "}"],
                                        dtype="S",
                                        order="C",
                                    ),
                                    [
                                        [
                                            "Transform",
                                            numpy.array(
                                                [1, 2, 3], dtype="int32", order="C"
                                            ),
                                            [],
                                            '"int[IndexDimension]"',
                                        ],
                                        [
                                            "PointRange",
                                            numpy.array(
                                                [[13, 13], [1, 61], [1, 17]],
                                                dtype="int32",
                                                order="F",
                                            ),
                                            [],
                                            "IndexRange_t",
                                        ],
                                        [
                                            "PointRangeDonor",
                                            numpy.array(
                                                [[1, 1], [1, 61], [1, 17]],
                                                dtype="int32",
                                                order="F",
                                            ),
                                            [],
                                            "IndexRange_t",
                                        ],
                                    ],
                                    "GridConnectivity1to1_t",
                                ],
                            ],
                            "ZoneGridConnectivity_t",
                        ],
                        [
                            "FlowSolution#EndOfRun",
                            None,
                            [
                                [
                                    "GridLocation",
                                    numpy.array(
                                        [
                                            "C",
                                            "e",
                                            "l",
                                            "l",
                                            "C",
                                            "e",
                                            "n",
                                            "t",
                                            "e",
                                            "r",
                                        ],
                                        dtype="S",
                                        order="C",
                                    ),
                                    [],
                                    "GridLocation_t",
                                ],
                                ["Density", None, [], "DataArray_t"],
                                ["MomentumX", None, [], "DataArray_t"],
                                ["MomentumY", None, [], "DataArray_t"],
                                ["MomentumZ", None, [], "DataArray_t"],
                                ["EnergyStagnationDensity", None, [], "DataArray_t"],
                            ],
                            "FlowSolution_t",
                        ],
                    ],
                    "Zone_t",
                ],
                [
                    "{Zone-B}",
                    numpy.array(
                        [[31, 30, 0], [61, 60, 0], [17, 16, 0]],
                        dtype="int64",
                        order="F",
                    ),
                    [
                        [
                            "ZoneType",
                            numpy.array(
                                ["S", "t", "r", "u", "c", "t", "u", "r", "e", "d"],
                                dtype="S",
                                order="C",
                            ),
                            [],
                            "ZoneType_t",
                        ],
                        [
                            "GridCoordinates",
                            None,
                            [
                                ["CoordinateX", None, [], "DataArray_t"],
                                ["CoordinateY", None, [], "DataArray_t"],
                                ["CoordinateZ", None, [], "DataArray_t"],
                            ],
                            "GridCoordinates_t",
                        ],
                        [
                            "ZoneBC",
                            None,
                            [
                                [
                                    "{BC-2}",
                                    numpy.array(
                                        [
                                            "F",
                                            "a",
                                            "m",
                                            "i",
                                            "l",
                                            "y",
                                            "S",
                                            "p",
                                            "e",
                                            "c",
                                            "i",
                                            "f",
                                            "i",
                                            "e",
                                            "d",
                                        ],
                                        dtype="S",
                                        order="C",
                                    ),
                                    [
                                        [
                                            "PointRange",
                                            numpy.array(
                                                [[1, 31], [1, 1], [1, 17]],
                                                dtype="int32",
                                                order="F",
                                            ),
                                            [],
                                            "IndexRange_t",
                                        ],
                                        [
                                            "FamilyName",
                                            numpy.array(
                                                [
                                                    "I",
                                                    "n",
                                                    "t",
                                                    "e",
                                                    "r",
                                                    "i",
                                                    "o",
                                                    "r",
                                                ],
                                                dtype="S",
                                                order="C",
                                            ),
                                            [],
                                            "FamilyName_t",
                                        ],
                                    ],
                                    "BC_t",
                                ],
                                [
                                    "{BC-1}",
                                    numpy.array(
                                        [
                                            "F",
                                            "a",
                                            "m",
                                            "i",
                                            "l",
                                            "y",
                                            "S",
                                            "p",
                                            "e",
                                            "c",
                                            "i",
                                            "f",
                                            "i",
                                            "e",
                                            "d",
                                        ],
                                        dtype="S",
                                        order="C",
                                    ),
                                    [
                                        [
                                            "PointRange",
                                            numpy.array(
                                                [[1, 31], [1, 61], [1, 1]],
                                                dtype="int32",
                                                order="F",
                                            ),
                                            [],
                                            "IndexRange_t",
                                        ],
                                        [
                                            "FamilyName",
                                            numpy.array(
                                                [
                                                    "E",
                                                    "x",
                                                    "t",
                                                    "e",
                                                    "r",
                                                    "i",
                                                    "o",
                                                    "r",
                                                ],
                                                dtype="S",
                                                order="C",
                                            ),
                                            [],
                                            "FamilyName_t",
                                        ],
                                    ],
                                    "BC_t",
                                ],
                                [
                                    "{BC-4}",
                                    numpy.array(
                                        [
                                            "F",
                                            "a",
                                            "m",
                                            "i",
                                            "l",
                                            "y",
                                            "S",
                                            "p",
                                            "e",
                                            "c",
                                            "i",
                                            "f",
                                            "i",
                                            "e",
                                            "d",
                                        ],
                                        dtype="S",
                                        order="C",
                                    ),
                                    [
                                        [
                                            "PointRange",
                                            numpy.array(
                                                [[1, 31], [61, 61], [1, 17]],
                                                dtype="int32",
                                                order="F",
                                            ),
                                            [],
                                            "IndexRange_t",
                                        ],
                                        [
                                            "FamilyName",
                                            numpy.array(
                                                [
                                                    "E",
                                                    "x",
                                                    "t",
                                                    "e",
                                                    "r",
                                                    "i",
                                                    "o",
                                                    "r",
                                                ],
                                                dtype="S",
                                                order="C",
                                            ),
                                            [],
                                            "FamilyName_t",
                                        ],
                                    ],
                                    "BC_t",
                                ],
                                [
                                    "{BC-6}",
                                    numpy.array(
                                        [
                                            "F",
                                            "a",
                                            "m",
                                            "i",
                                            "l",
                                            "y",
                                            "S",
                                            "p",
                                            "e",
                                            "c",
                                            "i",
                                            "f",
                                            "i",
                                            "e",
                                            "d",
                                        ],
                                        dtype="S",
                                        order="C",
                                    ),
                                    [
                                        [
                                            "PointRange",
                                            numpy.array(
                                                [[1, 31], [1, 61], [17, 17]],
                                                dtype="int32",
                                                order="F",
                                            ),
                                            [],
                                            "IndexRange_t",
                                        ],
                                        [
                                            "FamilyName",
                                            numpy.array(
                                                [
                                                    "E",
                                                    "x",
                                                    "t",
                                                    "e",
                                                    "r",
                                                    "i",
                                                    "o",
                                                    "r",
                                                ],
                                                dtype="S",
                                                order="C",
                                            ),
                                            [],
                                            "FamilyName_t",
                                        ],
                                    ],
                                    "BC_t",
                                ],
                            ],
                            "ZoneBC_t",
                        ],
                        [
                            "ZoneGridConnectivity",
                            None,
                            [
                                [
                                    "{CT-B-A}",
                                    numpy.array(
                                        ["{", "Z", "o", "n", "e", "-", "A", "}"],
                                        dtype="S",
                                        order="C",
                                    ),
                                    [
                                        [
                                            "Transform",
                                            numpy.array(
                                                [1, 2, 3], dtype="int32", order="C"
                                            ),
                                            [],
                                            '"int[IndexDimension]"',
                                        ],
                                        [
                                            "PointRange",
                                            numpy.array(
                                                [[1, 1], [1, 61], [1, 17]],
                                                dtype="int32",
                                                order="F",
                                            ),
                                            [],
                                            "IndexRange_t",
                                        ],
                                        [
                                            "PointRangeDonor",
                                            numpy.array(
                                                [[13, 13], [1, 61], [1, 17]],
                                                dtype="int32",
                                                order="F",
                                            ),
                                            [],
                                            "IndexRange_t",
                                        ],
                                    ],
                                    "GridConnectivity1to1_t",
                                ],
                                [
                                    "{CT-B-C}",
                                    numpy.array(
                                        ["{", "Z", "o", "n", "e", "-", "C", "}"],
                                        dtype="S",
                                        order="C",
                                    ),
                                    [
                                        [
                                            "GridConnectivityType",
                                            numpy.array(
                                                [
                                                    "A",
                                                    "b",
                                                    "u",
                                                    "t",
                                                    "t",
                                                    "i",
                                                    "n",
                                                    "g",
                                                ],
                                                dtype="S",
                                                order="C",
                                            ),
                                            [],
                                            "GridConnectivityType_t",
                                        ],
                                        [
                                            "ConnectivityKeyLocal",
                                            numpy.array(
                                                ["B", "_", "C"], dtype="S", order="C"
                                            ),
                                            [],
                                            "DataArray_t",
                                        ],
                                        [
                                            "ConnectivityKeyDonor",
                                            numpy.array(
                                                ["C", "_", "B"], dtype="S", order="C"
                                            ),
                                            [],
                                            "DataArray_t",
                                        ],
                                        [
                                            "PointRange",
                                            numpy.array(
                                                [[31, 31], [1, 61], [1, 17]],
                                                dtype="int32",
                                                order="F",
                                            ),
                                            [],
                                            "IndexRange_t",
                                        ],
                                        [
                                            ".Solver#Property",
                                            None,
                                            [
                                                [
                                                    "globborder",
                                                    numpy.array(
                                                        ["B", "_", "C"],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                                [
                                                    "globborderdonor",
                                                    numpy.array(
                                                        ["C", "_", "B"],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                                [
                                                    "jtype",
                                                    numpy.array(
                                                        [
                                                            "n",
                                                            "o",
                                                            "m",
                                                            "a",
                                                            "t",
                                                            "c",
                                                            "h",
                                                        ],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                                [
                                                    "nomatch_special",
                                                    numpy.array(
                                                        ["n", "o", "n", "e"],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                                [
                                                    "type",
                                                    numpy.array(
                                                        ["j", "o", "i", "n"],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                            ],
                                            "UserDefinedData_t",
                                        ],
                                    ],
                                    "GridConnectivity_t",
                                ],
                            ],
                            "ZoneGridConnectivity_t",
                        ],
                        [
                            "FlowSolution#EndOfRun",
                            None,
                            [
                                [
                                    "GridLocation",
                                    numpy.array(
                                        [
                                            "C",
                                            "e",
                                            "l",
                                            "l",
                                            "C",
                                            "e",
                                            "n",
                                            "t",
                                            "e",
                                            "r",
                                        ],
                                        dtype="S",
                                        order="C",
                                    ),
                                    [],
                                    "GridLocation_t",
                                ],
                                ["Density", None, [], "DataArray_t"],
                                ["MomentumX", None, [], "DataArray_t"],
                                ["MomentumY", None, [], "DataArray_t"],
                                ["MomentumZ", None, [], "DataArray_t"],
                                ["EnergyStagnationDensity", None, [], "DataArray_t"],
                            ],
                            "FlowSolution_t",
                        ],
                    ],
                    "Zone_t",
                ],
                [
                    "{Zone-C1}",
                    numpy.array(
                        [[27, 26, 0], [25, 24, 0], [17, 16, 0]],
                        dtype="int64",
                        order="F",
                    ),
                    [
                        [
                            "ZoneType",
                            numpy.array(
                                ["S", "t", "r", "u", "c", "t", "u", "r", "e", "d"],
                                dtype="S",
                                order="C",
                            ),
                            [],
                            "ZoneType_t",
                        ],
                        [
                            "GridCoordinates",
                            None,
                            [
                                ["CoordinateX", None, [], "DataArray_t"],
                                ["CoordinateY", None, [], "DataArray_t"],
                                ["CoordinateZ", None, [], "DataArray_t"],
                            ],
                            "GridCoordinates_t",
                        ],
                        [
                            "ZoneBC",
                            None,
                            [
                                [
                                    "{BC-1}",
                                    numpy.array(
                                        [
                                            "F",
                                            "a",
                                            "m",
                                            "i",
                                            "l",
                                            "y",
                                            "S",
                                            "p",
                                            "e",
                                            "c",
                                            "i",
                                            "f",
                                            "i",
                                            "e",
                                            "d",
                                        ],
                                        dtype="S",
                                        order="C",
                                    ),
                                    [
                                        [
                                            "PointRange",
                                            numpy.array(
                                                [[1, 27], [1, 25], [1, 1]],
                                                dtype="int32",
                                                order="F",
                                            ),
                                            [],
                                            "IndexRange_t",
                                        ],
                                        [
                                            "FamilyName",
                                            numpy.array(
                                                [
                                                    "E",
                                                    "x",
                                                    "t",
                                                    "e",
                                                    "r",
                                                    "i",
                                                    "o",
                                                    "r",
                                                ],
                                                dtype="S",
                                                order="C",
                                            ),
                                            [],
                                            "FamilyName_t",
                                        ],
                                    ],
                                    "BC_t",
                                ],
                                [
                                    "{BC-4}",
                                    numpy.array(
                                        [
                                            "F",
                                            "a",
                                            "m",
                                            "i",
                                            "l",
                                            "y",
                                            "S",
                                            "p",
                                            "e",
                                            "c",
                                            "i",
                                            "f",
                                            "i",
                                            "e",
                                            "d",
                                        ],
                                        dtype="S",
                                        order="C",
                                    ),
                                    [
                                        [
                                            "PointRange",
                                            numpy.array(
                                                [[1, 27], [25, 25], [1, 17]],
                                                dtype="int32",
                                                order="F",
                                            ),
                                            [],
                                            "IndexRange_t",
                                        ],
                                        [
                                            "FamilyName",
                                            numpy.array(
                                                [
                                                    "E",
                                                    "x",
                                                    "t",
                                                    "e",
                                                    "r",
                                                    "i",
                                                    "o",
                                                    "r",
                                                ],
                                                dtype="S",
                                                order="C",
                                            ),
                                            [],
                                            "FamilyName_t",
                                        ],
                                    ],
                                    "BC_t",
                                ],
                                [
                                    "{BC-6}",
                                    numpy.array(
                                        [
                                            "F",
                                            "a",
                                            "m",
                                            "i",
                                            "l",
                                            "y",
                                            "S",
                                            "p",
                                            "e",
                                            "c",
                                            "i",
                                            "f",
                                            "i",
                                            "e",
                                            "d",
                                        ],
                                        dtype="S",
                                        order="C",
                                    ),
                                    [
                                        [
                                            "PointRange",
                                            numpy.array(
                                                [[1, 27], [1, 25], [17, 17]],
                                                dtype="int32",
                                                order="F",
                                            ),
                                            [],
                                            "IndexRange_t",
                                        ],
                                        [
                                            "FamilyName",
                                            numpy.array(
                                                [
                                                    "E",
                                                    "x",
                                                    "t",
                                                    "e",
                                                    "r",
                                                    "i",
                                                    "o",
                                                    "r",
                                                ],
                                                dtype="S",
                                                order="C",
                                            ),
                                            [],
                                            "FamilyName_t",
                                        ],
                                    ],
                                    "BC_t",
                                ],
                            ],
                            "ZoneBC_t",
                        ],
                        [
                            "ZoneGridConnectivity",
                            None,
                            [
                                [
                                    "{CT-C1-C2}",
                                    numpy.array(
                                        ["{", "Z", "o", "n", "e", "-", "C", "2", "}"],
                                        dtype="S",
                                        order="C",
                                    ),
                                    [
                                        [
                                            "Transform",
                                            numpy.array(
                                                [1, 2, 3], dtype="int32", order="C"
                                            ),
                                            [],
                                            '"int[IndexDimension]"',
                                        ],
                                        [
                                            "PointRange",
                                            numpy.array(
                                                [[1, 27], [1, 1], [1, 17]],
                                                dtype="int32",
                                                order="F",
                                            ),
                                            [],
                                            "IndexRange_t",
                                        ],
                                        [
                                            "PointRangeDonor",
                                            numpy.array(
                                                [[1, 27], [37, 37], [1, 17]],
                                                dtype="int32",
                                                order="F",
                                            ),
                                            [],
                                            "IndexRange_t",
                                        ],
                                    ],
                                    "GridConnectivity1to1_t",
                                ],
                                [
                                    "{CT-C1-B}",
                                    numpy.array(
                                        ["{", "Z", "o", "n", "e", "-", "B", "}"],
                                        dtype="S",
                                        order="C",
                                    ),
                                    [
                                        [
                                            "GridConnectivityType",
                                            numpy.array(
                                                [
                                                    "A",
                                                    "b",
                                                    "u",
                                                    "t",
                                                    "t",
                                                    "i",
                                                    "n",
                                                    "g",
                                                ],
                                                dtype="S",
                                                order="C",
                                            ),
                                            [],
                                            "GridConnectivityType_t",
                                        ],
                                        [
                                            "ConnectivityKeyLocal",
                                            numpy.array(
                                                ["C", "_", "B"], dtype="S", order="C"
                                            ),
                                            [],
                                            "DataArray_t",
                                        ],
                                        [
                                            "ConnectivityKeyDonor",
                                            numpy.array(
                                                ["B", "_", "C"], dtype="S", order="C"
                                            ),
                                            [],
                                            "DataArray_t",
                                        ],
                                        [
                                            "PointRange",
                                            numpy.array(
                                                [[1, 1], [1, 25], [1, 17]],
                                                dtype="int32",
                                                order="F",
                                            ),
                                            [],
                                            "IndexRange_t",
                                        ],
                                        [
                                            ".Solver#Property",
                                            None,
                                            [
                                                [
                                                    "globborder",
                                                    numpy.array(
                                                        ["C", "_", "B"],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                                [
                                                    "globborderdonor",
                                                    numpy.array(
                                                        ["B", "_", "C"],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                                [
                                                    "jtype",
                                                    numpy.array(
                                                        [
                                                            "n",
                                                            "o",
                                                            "m",
                                                            "a",
                                                            "t",
                                                            "c",
                                                            "h",
                                                        ],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                                [
                                                    "nomatch_special",
                                                    numpy.array(
                                                        ["n", "o", "n", "e"],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                                [
                                                    "type",
                                                    numpy.array(
                                                        ["j", "o", "i", "n"],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                            ],
                                            "UserDefinedData_t",
                                        ],
                                    ],
                                    "GridConnectivity_t",
                                ],
                                [
                                    "{CT-C1-D}",
                                    numpy.array(
                                        ["{", "Z", "o", "n", "e", "-", "D", "}"],
                                        dtype="S",
                                        order="C",
                                    ),
                                    [
                                        [
                                            "GridConnectivityType",
                                            numpy.array(
                                                [
                                                    "A",
                                                    "b",
                                                    "u",
                                                    "t",
                                                    "t",
                                                    "i",
                                                    "n",
                                                    "g",
                                                ],
                                                dtype="S",
                                                order="C",
                                            ),
                                            [],
                                            "GridConnectivityType_t",
                                        ],
                                        [
                                            "ConnectivityKeyLocal",
                                            numpy.array(
                                                ["C", "_", "D"], dtype="S", order="C"
                                            ),
                                            [],
                                            "DataArray_t",
                                        ],
                                        [
                                            "ConnectivityKeyDonor",
                                            numpy.array(
                                                ["D", "_", "C"], dtype="S", order="C"
                                            ),
                                            [],
                                            "DataArray_t",
                                        ],
                                        [
                                            "PointRange",
                                            numpy.array(
                                                [[27, 27], [1, 25], [1, 17]],
                                                dtype="int32",
                                                order="F",
                                            ),
                                            [],
                                            "IndexRange_t",
                                        ],
                                        [
                                            ".Solver#Property",
                                            None,
                                            [
                                                [
                                                    "globborder",
                                                    numpy.array(
                                                        ["C", "_", "D"],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                                [
                                                    "globborderdonor",
                                                    numpy.array(
                                                        ["D", "_", "C"],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                                [
                                                    "jtype",
                                                    numpy.array(
                                                        [
                                                            "n",
                                                            "o",
                                                            "m",
                                                            "a",
                                                            "t",
                                                            "c",
                                                            "h",
                                                        ],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                                [
                                                    "nomatch_special",
                                                    numpy.array(
                                                        ["n", "o", "n", "e"],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                                [
                                                    "type",
                                                    numpy.array(
                                                        ["j", "o", "i", "n"],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                            ],
                                            "UserDefinedData_t",
                                        ],
                                    ],
                                    "GridConnectivity_t",
                                ],
                            ],
                            "ZoneGridConnectivity_t",
                        ],
                        [
                            "FlowSolution#EndOfRun",
                            None,
                            [
                                [
                                    "GridLocation",
                                    numpy.array(
                                        [
                                            "C",
                                            "e",
                                            "l",
                                            "l",
                                            "C",
                                            "e",
                                            "n",
                                            "t",
                                            "e",
                                            "r",
                                        ],
                                        dtype="S",
                                        order="C",
                                    ),
                                    [],
                                    "GridLocation_t",
                                ],
                                ["Density", None, [], "DataArray_t"],
                                ["MomentumX", None, [], "DataArray_t"],
                                ["MomentumY", None, [], "DataArray_t"],
                                ["MomentumZ", None, [], "DataArray_t"],
                                ["EnergyStagnationDensity", None, [], "DataArray_t"],
                            ],
                            "FlowSolution_t",
                        ],
                    ],
                    "Zone_t",
                ],
                [
                    "{Zone-C2}",
                    numpy.array(
                        [[27, 26, 0], [37, 36, 0], [17, 16, 0]],
                        dtype="int64",
                        order="F",
                    ),
                    [
                        [
                            "ZoneType",
                            numpy.array(
                                ["S", "t", "r", "u", "c", "t", "u", "r", "e", "d"],
                                dtype="S",
                                order="C",
                            ),
                            [],
                            "ZoneType_t",
                        ],
                        [
                            "GridCoordinates",
                            None,
                            [
                                ["CoordinateX", None, [], "DataArray_t"],
                                ["CoordinateY", None, [], "DataArray_t"],
                                ["CoordinateZ", None, [], "DataArray_t"],
                            ],
                            "GridCoordinates_t",
                        ],
                        [
                            "ZoneBC",
                            None,
                            [
                                [
                                    "{BC-2}",
                                    numpy.array(
                                        [
                                            "F",
                                            "a",
                                            "m",
                                            "i",
                                            "l",
                                            "y",
                                            "S",
                                            "p",
                                            "e",
                                            "c",
                                            "i",
                                            "f",
                                            "i",
                                            "e",
                                            "d",
                                        ],
                                        dtype="S",
                                        order="C",
                                    ),
                                    [
                                        [
                                            "PointRange",
                                            numpy.array(
                                                [[1, 27], [1, 1], [1, 17]],
                                                dtype="int32",
                                                order="F",
                                            ),
                                            [],
                                            "IndexRange_t",
                                        ],
                                        [
                                            "FamilyName",
                                            numpy.array(
                                                [
                                                    "I",
                                                    "n",
                                                    "t",
                                                    "e",
                                                    "r",
                                                    "i",
                                                    "o",
                                                    "r",
                                                ],
                                                dtype="S",
                                                order="C",
                                            ),
                                            [],
                                            "FamilyName_t",
                                        ],
                                    ],
                                    "BC_t",
                                ],
                                [
                                    "{BC-1}",
                                    numpy.array(
                                        [
                                            "F",
                                            "a",
                                            "m",
                                            "i",
                                            "l",
                                            "y",
                                            "S",
                                            "p",
                                            "e",
                                            "c",
                                            "i",
                                            "f",
                                            "i",
                                            "e",
                                            "d",
                                        ],
                                        dtype="S",
                                        order="C",
                                    ),
                                    [
                                        [
                                            "PointRange",
                                            numpy.array(
                                                [[1, 27], [1, 37], [1, 1]],
                                                dtype="int32",
                                                order="F",
                                            ),
                                            [],
                                            "IndexRange_t",
                                        ],
                                        [
                                            "FamilyName",
                                            numpy.array(
                                                [
                                                    "E",
                                                    "x",
                                                    "t",
                                                    "e",
                                                    "r",
                                                    "i",
                                                    "o",
                                                    "r",
                                                ],
                                                dtype="S",
                                                order="C",
                                            ),
                                            [],
                                            "FamilyName_t",
                                        ],
                                    ],
                                    "BC_t",
                                ],
                                [
                                    "{BC-6}",
                                    numpy.array(
                                        [
                                            "F",
                                            "a",
                                            "m",
                                            "i",
                                            "l",
                                            "y",
                                            "S",
                                            "p",
                                            "e",
                                            "c",
                                            "i",
                                            "f",
                                            "i",
                                            "e",
                                            "d",
                                        ],
                                        dtype="S",
                                        order="C",
                                    ),
                                    [
                                        [
                                            "PointRange",
                                            numpy.array(
                                                [[1, 27], [1, 37], [17, 17]],
                                                dtype="int32",
                                                order="F",
                                            ),
                                            [],
                                            "IndexRange_t",
                                        ],
                                        [
                                            "FamilyName",
                                            numpy.array(
                                                [
                                                    "E",
                                                    "x",
                                                    "t",
                                                    "e",
                                                    "r",
                                                    "i",
                                                    "o",
                                                    "r",
                                                ],
                                                dtype="S",
                                                order="C",
                                            ),
                                            [],
                                            "FamilyName_t",
                                        ],
                                    ],
                                    "BC_t",
                                ],
                            ],
                            "ZoneBC_t",
                        ],
                        [
                            "ZoneGridConnectivity",
                            None,
                            [
                                [
                                    "{CT-C2-B}",
                                    numpy.array(
                                        ["{", "Z", "o", "n", "e", "-", "B", "}"],
                                        dtype="S",
                                        order="C",
                                    ),
                                    [
                                        [
                                            "GridConnectivityType",
                                            numpy.array(
                                                [
                                                    "A",
                                                    "b",
                                                    "u",
                                                    "t",
                                                    "t",
                                                    "i",
                                                    "n",
                                                    "g",
                                                ],
                                                dtype="S",
                                                order="C",
                                            ),
                                            [],
                                            "GridConnectivityType_t",
                                        ],
                                        [
                                            "ConnectivityKeyLocal",
                                            numpy.array(
                                                ["C", "_", "B"], dtype="S", order="C"
                                            ),
                                            [],
                                            "DataArray_t",
                                        ],
                                        [
                                            "ConnectivityKeyDonor",
                                            numpy.array(
                                                ["B", "_", "C"], dtype="S", order="C"
                                            ),
                                            [],
                                            "DataArray_t",
                                        ],
                                        [
                                            "PointRange",
                                            numpy.array(
                                                [[1, 1], [1, 37], [1, 17]],
                                                dtype="int32",
                                                order="F",
                                            ),
                                            [],
                                            "IndexRange_t",
                                        ],
                                        [
                                            ".Solver#Property",
                                            None,
                                            [
                                                [
                                                    "globborder",
                                                    numpy.array(
                                                        ["C", "_", "B"],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                                [
                                                    "globborderdonor",
                                                    numpy.array(
                                                        ["B", "_", "C"],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                                [
                                                    "jtype",
                                                    numpy.array(
                                                        [
                                                            "n",
                                                            "o",
                                                            "m",
                                                            "a",
                                                            "t",
                                                            "c",
                                                            "h",
                                                        ],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                                [
                                                    "nomatch_special",
                                                    numpy.array(
                                                        ["n", "o", "n", "e"],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                                [
                                                    "type",
                                                    numpy.array(
                                                        ["j", "o", "i", "n"],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                            ],
                                            "UserDefinedData_t",
                                        ],
                                    ],
                                    "GridConnectivity_t",
                                ],
                                [
                                    "{CT-C2-D}",
                                    numpy.array(
                                        ["{", "Z", "o", "n", "e", "-", "D", "}"],
                                        dtype="S",
                                        order="C",
                                    ),
                                    [
                                        [
                                            "GridConnectivityType",
                                            numpy.array(
                                                [
                                                    "A",
                                                    "b",
                                                    "u",
                                                    "t",
                                                    "t",
                                                    "i",
                                                    "n",
                                                    "g",
                                                ],
                                                dtype="S",
                                                order="C",
                                            ),
                                            [],
                                            "GridConnectivityType_t",
                                        ],
                                        [
                                            "ConnectivityKeyLocal",
                                            numpy.array(
                                                ["C", "_", "D"], dtype="S", order="C"
                                            ),
                                            [],
                                            "DataArray_t",
                                        ],
                                        [
                                            "ConnectivityKeyDonor",
                                            numpy.array(
                                                ["D", "_", "C"], dtype="S", order="C"
                                            ),
                                            [],
                                            "DataArray_t",
                                        ],
                                        [
                                            "PointRange",
                                            numpy.array(
                                                [[27, 27], [1, 37], [1, 17]],
                                                dtype="int32",
                                                order="F",
                                            ),
                                            [],
                                            "IndexRange_t",
                                        ],
                                        [
                                            ".Solver#Property",
                                            None,
                                            [
                                                [
                                                    "globborder",
                                                    numpy.array(
                                                        ["C", "_", "D"],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                                [
                                                    "globborderdonor",
                                                    numpy.array(
                                                        ["D", "_", "C"],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                                [
                                                    "jtype",
                                                    numpy.array(
                                                        [
                                                            "n",
                                                            "o",
                                                            "m",
                                                            "a",
                                                            "t",
                                                            "c",
                                                            "h",
                                                        ],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                                [
                                                    "nomatch_special",
                                                    numpy.array(
                                                        ["n", "o", "n", "e"],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                                [
                                                    "type",
                                                    numpy.array(
                                                        ["j", "o", "i", "n"],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                            ],
                                            "UserDefinedData_t",
                                        ],
                                    ],
                                    "GridConnectivity_t",
                                ],
                                [
                                    "{CT-C2-C1}",
                                    numpy.array(
                                        ["{", "Z", "o", "n", "e", "-", "C", "1", "}"],
                                        dtype="S",
                                        order="C",
                                    ),
                                    [
                                        [
                                            "Transform",
                                            numpy.array(
                                                [1, 2, 3], dtype="int32", order="C"
                                            ),
                                            [],
                                            '"int[IndexDimension]"',
                                        ],
                                        [
                                            "PointRange",
                                            numpy.array(
                                                [[1, 27], [37, 37], [1, 17]],
                                                dtype="int32",
                                                order="F",
                                            ),
                                            [],
                                            "IndexRange_t",
                                        ],
                                        [
                                            "PointRangeDonor",
                                            numpy.array(
                                                [[1, 27], [1, 1], [1, 17]],
                                                dtype="int32",
                                                order="F",
                                            ),
                                            [],
                                            "IndexRange_t",
                                        ],
                                    ],
                                    "GridConnectivity1to1_t",
                                ],
                            ],
                            "ZoneGridConnectivity_t",
                        ],
                        [
                            "FlowSolution#EndOfRun",
                            None,
                            [
                                [
                                    "GridLocation",
                                    numpy.array(
                                        [
                                            "C",
                                            "e",
                                            "l",
                                            "l",
                                            "C",
                                            "e",
                                            "n",
                                            "t",
                                            "e",
                                            "r",
                                        ],
                                        dtype="S",
                                        order="C",
                                    ),
                                    [],
                                    "GridLocation_t",
                                ],
                                ["Density", None, [], "DataArray_t"],
                                ["MomentumX", None, [], "DataArray_t"],
                                ["MomentumY", None, [], "DataArray_t"],
                                ["MomentumZ", None, [], "DataArray_t"],
                                ["EnergyStagnationDensity", None, [], "DataArray_t"],
                            ],
                            "FlowSolution_t",
                        ],
                    ],
                    "Zone_t",
                ],
                [
                    "{Zone-D1}",
                    numpy.array(
                        [[41, 40, 0], [61, 60, 0], [17, 16, 0]],
                        dtype="int64",
                        order="F",
                    ),
                    [
                        [
                            "ZoneType",
                            numpy.array(
                                ["S", "t", "r", "u", "c", "t", "u", "r", "e", "d"],
                                dtype="S",
                                order="C",
                            ),
                            [],
                            "ZoneType_t",
                        ],
                        [
                            "GridCoordinates",
                            None,
                            [
                                ["CoordinateX", None, [], "DataArray_t"],
                                ["CoordinateY", None, [], "DataArray_t"],
                                ["CoordinateZ", None, [], "DataArray_t"],
                            ],
                            "GridCoordinates_t",
                        ],
                        [
                            "ZoneBC",
                            None,
                            [
                                [
                                    "{BC-2}",
                                    numpy.array(
                                        [
                                            "F",
                                            "a",
                                            "m",
                                            "i",
                                            "l",
                                            "y",
                                            "S",
                                            "p",
                                            "e",
                                            "c",
                                            "i",
                                            "f",
                                            "i",
                                            "e",
                                            "d",
                                        ],
                                        dtype="S",
                                        order="C",
                                    ),
                                    [
                                        [
                                            "PointRange",
                                            numpy.array(
                                                [[1, 41], [1, 1], [1, 17]],
                                                dtype="int32",
                                                order="F",
                                            ),
                                            [],
                                            "IndexRange_t",
                                        ],
                                        [
                                            "FamilyName",
                                            numpy.array(
                                                [
                                                    "I",
                                                    "n",
                                                    "t",
                                                    "e",
                                                    "r",
                                                    "i",
                                                    "o",
                                                    "r",
                                                ],
                                                dtype="S",
                                                order="C",
                                            ),
                                            [],
                                            "FamilyName_t",
                                        ],
                                    ],
                                    "BC_t",
                                ],
                                [
                                    "{BC-4}",
                                    numpy.array(
                                        [
                                            "F",
                                            "a",
                                            "m",
                                            "i",
                                            "l",
                                            "y",
                                            "S",
                                            "p",
                                            "e",
                                            "c",
                                            "i",
                                            "f",
                                            "i",
                                            "e",
                                            "d",
                                        ],
                                        dtype="S",
                                        order="C",
                                    ),
                                    [
                                        [
                                            "PointRange",
                                            numpy.array(
                                                [[1, 41], [61, 61], [1, 17]],
                                                dtype="int32",
                                                order="F",
                                            ),
                                            [],
                                            "IndexRange_t",
                                        ],
                                        [
                                            "FamilyName",
                                            numpy.array(
                                                [
                                                    "E",
                                                    "x",
                                                    "t",
                                                    "e",
                                                    "r",
                                                    "i",
                                                    "o",
                                                    "r",
                                                ],
                                                dtype="S",
                                                order="C",
                                            ),
                                            [],
                                            "FamilyName_t",
                                        ],
                                    ],
                                    "BC_t",
                                ],
                                [
                                    "{BC-6}",
                                    numpy.array(
                                        [
                                            "F",
                                            "a",
                                            "m",
                                            "i",
                                            "l",
                                            "y",
                                            "S",
                                            "p",
                                            "e",
                                            "c",
                                            "i",
                                            "f",
                                            "i",
                                            "e",
                                            "d",
                                        ],
                                        dtype="S",
                                        order="C",
                                    ),
                                    [
                                        [
                                            "PointRange",
                                            numpy.array(
                                                [[1, 41], [1, 61], [17, 17]],
                                                dtype="int32",
                                                order="F",
                                            ),
                                            [],
                                            "IndexRange_t",
                                        ],
                                        [
                                            "FamilyName",
                                            numpy.array(
                                                [
                                                    "E",
                                                    "x",
                                                    "t",
                                                    "e",
                                                    "r",
                                                    "i",
                                                    "o",
                                                    "r",
                                                ],
                                                dtype="S",
                                                order="C",
                                            ),
                                            [],
                                            "FamilyName_t",
                                        ],
                                    ],
                                    "BC_t",
                                ],
                            ],
                            "ZoneBC_t",
                        ],
                        [
                            "ZoneGridConnectivity",
                            None,
                            [
                                [
                                    "{CT-D1-C}",
                                    numpy.array(
                                        ["{", "Z", "o", "n", "e", "-", "C", "}"],
                                        dtype="S",
                                        order="C",
                                    ),
                                    [
                                        [
                                            "GridConnectivityType",
                                            numpy.array(
                                                [
                                                    "A",
                                                    "b",
                                                    "u",
                                                    "t",
                                                    "t",
                                                    "i",
                                                    "n",
                                                    "g",
                                                ],
                                                dtype="S",
                                                order="C",
                                            ),
                                            [],
                                            "GridConnectivityType_t",
                                        ],
                                        [
                                            "ConnectivityKeyLocal",
                                            numpy.array(
                                                ["D", "_", "C"], dtype="S", order="C"
                                            ),
                                            [],
                                            "DataArray_t",
                                        ],
                                        [
                                            "ConnectivityKeyDonor",
                                            numpy.array(
                                                ["C", "_", "D"], dtype="S", order="C"
                                            ),
                                            [],
                                            "DataArray_t",
                                        ],
                                        [
                                            "PointRange",
                                            numpy.array(
                                                [[1, 1], [1, 61], [1, 17]],
                                                dtype="int32",
                                                order="F",
                                            ),
                                            [],
                                            "IndexRange_t",
                                        ],
                                        [
                                            ".Solver#Property",
                                            None,
                                            [
                                                [
                                                    "globborder",
                                                    numpy.array(
                                                        ["D", "_", "C"],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                                [
                                                    "globborderdonor",
                                                    numpy.array(
                                                        ["C", "_", "D"],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                                [
                                                    "jtype",
                                                    numpy.array(
                                                        [
                                                            "n",
                                                            "o",
                                                            "m",
                                                            "a",
                                                            "t",
                                                            "c",
                                                            "h",
                                                        ],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                                [
                                                    "nomatch_special",
                                                    numpy.array(
                                                        ["n", "o", "n", "e"],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                                [
                                                    "type",
                                                    numpy.array(
                                                        ["j", "o", "i", "n"],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                            ],
                                            "UserDefinedData_t",
                                        ],
                                    ],
                                    "GridConnectivity_t",
                                ],
                                [
                                    "{CT-D1-A}",
                                    numpy.array(
                                        ["{", "Z", "o", "n", "e", "-", "A", "}"],
                                        dtype="S",
                                        order="C",
                                    ),
                                    [
                                        [
                                            "GridConnectivityType",
                                            numpy.array(
                                                [
                                                    "A",
                                                    "b",
                                                    "u",
                                                    "t",
                                                    "t",
                                                    "i",
                                                    "n",
                                                    "g",
                                                ],
                                                dtype="S",
                                                order="C",
                                            ),
                                            [],
                                            "GridConnectivityType_t",
                                        ],
                                        [
                                            "ConnectivityKeyLocal",
                                            numpy.array(
                                                ["D", "_", "A"], dtype="S", order="C"
                                            ),
                                            [],
                                            "DataArray_t",
                                        ],
                                        [
                                            "ConnectivityKeyDonor",
                                            numpy.array(
                                                ["A", "_", "D"], dtype="S", order="C"
                                            ),
                                            [],
                                            "DataArray_t",
                                        ],
                                        [
                                            "PointRange",
                                            numpy.array(
                                                [[41, 41], [1, 61], [1, 17]],
                                                dtype="int32",
                                                order="F",
                                            ),
                                            [],
                                            "IndexRange_t",
                                        ],
                                        [
                                            ".Solver#Property",
                                            None,
                                            [
                                                [
                                                    "globborder",
                                                    numpy.array(
                                                        ["D", "_", "A"],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                                [
                                                    "globborderdonor",
                                                    numpy.array(
                                                        ["A", "_", "D"],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                                [
                                                    "jtype",
                                                    numpy.array(
                                                        [
                                                            "n",
                                                            "o",
                                                            "m",
                                                            "a",
                                                            "t",
                                                            "c",
                                                            "h",
                                                        ],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                                [
                                                    "nomatch_special",
                                                    numpy.array(
                                                        ["n", "o", "n", "e"],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                                [
                                                    "type",
                                                    numpy.array(
                                                        ["j", "o", "i", "n"],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                            ],
                                            "UserDefinedData_t",
                                        ],
                                    ],
                                    "GridConnectivity_t",
                                ],
                                [
                                    "{CT-D1-D2}",
                                    numpy.array(
                                        ["{", "Z", "o", "n", "e", "-", "D", "2", "}"],
                                        dtype="S",
                                        order="C",
                                    ),
                                    [
                                        [
                                            "Transform",
                                            numpy.array(
                                                [1, 2, 3], dtype="int32", order="C"
                                            ),
                                            [],
                                            '"int[IndexDimension]"',
                                        ],
                                        [
                                            "PointRange",
                                            numpy.array(
                                                [[1, 41], [1, 61], [1, 1]],
                                                dtype="int32",
                                                order="F",
                                            ),
                                            [],
                                            "IndexRange_t",
                                        ],
                                        [
                                            "PointRangeDonor",
                                            numpy.array(
                                                [[1, 41], [1, 61], [17, 17]],
                                                dtype="int32",
                                                order="F",
                                            ),
                                            [],
                                            "IndexRange_t",
                                        ],
                                    ],
                                    "GridConnectivity1to1_t",
                                ],
                            ],
                            "ZoneGridConnectivity_t",
                        ],
                        [
                            "FlowSolution#EndOfRun",
                            None,
                            [
                                [
                                    "GridLocation",
                                    numpy.array(
                                        [
                                            "C",
                                            "e",
                                            "l",
                                            "l",
                                            "C",
                                            "e",
                                            "n",
                                            "t",
                                            "e",
                                            "r",
                                        ],
                                        dtype="S",
                                        order="C",
                                    ),
                                    [],
                                    "GridLocation_t",
                                ],
                                ["Density", None, [], "DataArray_t"],
                                ["MomentumX", None, [], "DataArray_t"],
                                ["MomentumY", None, [], "DataArray_t"],
                                ["MomentumZ", None, [], "DataArray_t"],
                                ["EnergyStagnationDensity", None, [], "DataArray_t"],
                            ],
                            "FlowSolution_t",
                        ],
                    ],
                    "Zone_t",
                ],
                [
                    "{Zone-D2}",
                    numpy.array(
                        [[41, 40, 0], [61, 60, 0], [17, 16, 0]],
                        dtype="int64",
                        order="F",
                    ),
                    [
                        [
                            "ZoneType",
                            numpy.array(
                                ["S", "t", "r", "u", "c", "t", "u", "r", "e", "d"],
                                dtype="S",
                                order="C",
                            ),
                            [],
                            "ZoneType_t",
                        ],
                        [
                            "GridCoordinates",
                            None,
                            [
                                ["CoordinateX", None, [], "DataArray_t"],
                                ["CoordinateY", None, [], "DataArray_t"],
                                ["CoordinateZ", None, [], "DataArray_t"],
                            ],
                            "GridCoordinates_t",
                        ],
                        [
                            "ZoneBC",
                            None,
                            [
                                [
                                    "{BC-2}",
                                    numpy.array(
                                        [
                                            "F",
                                            "a",
                                            "m",
                                            "i",
                                            "l",
                                            "y",
                                            "S",
                                            "p",
                                            "e",
                                            "c",
                                            "i",
                                            "f",
                                            "i",
                                            "e",
                                            "d",
                                        ],
                                        dtype="S",
                                        order="C",
                                    ),
                                    [
                                        [
                                            "PointRange",
                                            numpy.array(
                                                [[1, 41], [1, 1], [1, 17]],
                                                dtype="int32",
                                                order="F",
                                            ),
                                            [],
                                            "IndexRange_t",
                                        ],
                                        [
                                            "FamilyName",
                                            numpy.array(
                                                [
                                                    "I",
                                                    "n",
                                                    "t",
                                                    "e",
                                                    "r",
                                                    "i",
                                                    "o",
                                                    "r",
                                                ],
                                                dtype="S",
                                                order="C",
                                            ),
                                            [],
                                            "FamilyName_t",
                                        ],
                                    ],
                                    "BC_t",
                                ],
                                [
                                    "{BC-1}",
                                    numpy.array(
                                        [
                                            "F",
                                            "a",
                                            "m",
                                            "i",
                                            "l",
                                            "y",
                                            "S",
                                            "p",
                                            "e",
                                            "c",
                                            "i",
                                            "f",
                                            "i",
                                            "e",
                                            "d",
                                        ],
                                        dtype="S",
                                        order="C",
                                    ),
                                    [
                                        [
                                            "PointRange",
                                            numpy.array(
                                                [[1, 41], [1, 61], [1, 1]],
                                                dtype="int32",
                                                order="F",
                                            ),
                                            [],
                                            "IndexRange_t",
                                        ],
                                        [
                                            "FamilyName",
                                            numpy.array(
                                                [
                                                    "E",
                                                    "x",
                                                    "t",
                                                    "e",
                                                    "r",
                                                    "i",
                                                    "o",
                                                    "r",
                                                ],
                                                dtype="S",
                                                order="C",
                                            ),
                                            [],
                                            "FamilyName_t",
                                        ],
                                    ],
                                    "BC_t",
                                ],
                                [
                                    "{BC-4}",
                                    numpy.array(
                                        [
                                            "F",
                                            "a",
                                            "m",
                                            "i",
                                            "l",
                                            "y",
                                            "S",
                                            "p",
                                            "e",
                                            "c",
                                            "i",
                                            "f",
                                            "i",
                                            "e",
                                            "d",
                                        ],
                                        dtype="S",
                                        order="C",
                                    ),
                                    [
                                        [
                                            "PointRange",
                                            numpy.array(
                                                [[1, 41], [61, 61], [1, 17]],
                                                dtype="int32",
                                                order="F",
                                            ),
                                            [],
                                            "IndexRange_t",
                                        ],
                                        [
                                            "FamilyName",
                                            numpy.array(
                                                [
                                                    "E",
                                                    "x",
                                                    "t",
                                                    "e",
                                                    "r",
                                                    "i",
                                                    "o",
                                                    "r",
                                                ],
                                                dtype="S",
                                                order="C",
                                            ),
                                            [],
                                            "FamilyName_t",
                                        ],
                                    ],
                                    "BC_t",
                                ],
                            ],
                            "ZoneBC_t",
                        ],
                        [
                            "ZoneGridConnectivity",
                            None,
                            [
                                [
                                    "{CT-D2-C}",
                                    numpy.array(
                                        ["{", "Z", "o", "n", "e", "-", "C", "}"],
                                        dtype="S",
                                        order="C",
                                    ),
                                    [
                                        [
                                            "GridConnectivityType",
                                            numpy.array(
                                                [
                                                    "A",
                                                    "b",
                                                    "u",
                                                    "t",
                                                    "t",
                                                    "i",
                                                    "n",
                                                    "g",
                                                ],
                                                dtype="S",
                                                order="C",
                                            ),
                                            [],
                                            "GridConnectivityType_t",
                                        ],
                                        [
                                            "ConnectivityKeyLocal",
                                            numpy.array(
                                                ["D", "_", "C"], dtype="S", order="C"
                                            ),
                                            [],
                                            "DataArray_t",
                                        ],
                                        [
                                            "ConnectivityKeyDonor",
                                            numpy.array(
                                                ["C", "_", "D"], dtype="S", order="C"
                                            ),
                                            [],
                                            "DataArray_t",
                                        ],
                                        [
                                            "PointRange",
                                            numpy.array(
                                                [[1, 1], [1, 61], [1, 17]],
                                                dtype="int32",
                                                order="F",
                                            ),
                                            [],
                                            "IndexRange_t",
                                        ],
                                        [
                                            ".Solver#Property",
                                            None,
                                            [
                                                [
                                                    "globborder",
                                                    numpy.array(
                                                        ["D", "_", "C"],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                                [
                                                    "globborderdonor",
                                                    numpy.array(
                                                        ["C", "_", "D"],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                                [
                                                    "jtype",
                                                    numpy.array(
                                                        [
                                                            "n",
                                                            "o",
                                                            "m",
                                                            "a",
                                                            "t",
                                                            "c",
                                                            "h",
                                                        ],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                                [
                                                    "nomatch_special",
                                                    numpy.array(
                                                        ["n", "o", "n", "e"],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                                [
                                                    "type",
                                                    numpy.array(
                                                        ["j", "o", "i", "n"],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                            ],
                                            "UserDefinedData_t",
                                        ],
                                    ],
                                    "GridConnectivity_t",
                                ],
                                [
                                    "{CT-D2-D1}",
                                    numpy.array(
                                        ["{", "Z", "o", "n", "e", "-", "D", "1", "}"],
                                        dtype="S",
                                        order="C",
                                    ),
                                    [
                                        [
                                            "Transform",
                                            numpy.array(
                                                [1, 2, 3], dtype="int32", order="C"
                                            ),
                                            [],
                                            '"int[IndexDimension]"',
                                        ],
                                        [
                                            "PointRange",
                                            numpy.array(
                                                [[1, 41], [1, 61], [17, 17]],
                                                dtype="int32",
                                                order="F",
                                            ),
                                            [],
                                            "IndexRange_t",
                                        ],
                                        [
                                            "PointRangeDonor",
                                            numpy.array(
                                                [[1, 41], [1, 61], [1, 1]],
                                                dtype="int32",
                                                order="F",
                                            ),
                                            [],
                                            "IndexRange_t",
                                        ],
                                    ],
                                    "GridConnectivity1to1_t",
                                ],
                                [
                                    "{CT-D2-A}",
                                    numpy.array(
                                        ["{", "Z", "o", "n", "e", "-", "A", "}"],
                                        dtype="S",
                                        order="C",
                                    ),
                                    [
                                        [
                                            "GridConnectivityType",
                                            numpy.array(
                                                [
                                                    "A",
                                                    "b",
                                                    "u",
                                                    "t",
                                                    "t",
                                                    "i",
                                                    "n",
                                                    "g",
                                                ],
                                                dtype="S",
                                                order="C",
                                            ),
                                            [],
                                            "GridConnectivityType_t",
                                        ],
                                        [
                                            "ConnectivityKeyLocal",
                                            numpy.array(
                                                ["D", "_", "A"], dtype="S", order="C"
                                            ),
                                            [],
                                            "DataArray_t",
                                        ],
                                        [
                                            "ConnectivityKeyDonor",
                                            numpy.array(
                                                ["A", "_", "D"], dtype="S", order="C"
                                            ),
                                            [],
                                            "DataArray_t",
                                        ],
                                        [
                                            "PointRange",
                                            numpy.array(
                                                [[41, 41], [1, 61], [1, 17]],
                                                dtype="int32",
                                                order="F",
                                            ),
                                            [],
                                            "IndexRange_t",
                                        ],
                                        [
                                            ".Solver#Property",
                                            None,
                                            [
                                                [
                                                    "globborder",
                                                    numpy.array(
                                                        ["D", "_", "A"],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                                [
                                                    "globborderdonor",
                                                    numpy.array(
                                                        ["A", "_", "D"],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                                [
                                                    "jtype",
                                                    numpy.array(
                                                        [
                                                            "n",
                                                            "o",
                                                            "m",
                                                            "a",
                                                            "t",
                                                            "c",
                                                            "h",
                                                        ],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                                [
                                                    "nomatch_special",
                                                    numpy.array(
                                                        ["n", "o", "n", "e"],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                                [
                                                    "type",
                                                    numpy.array(
                                                        ["j", "o", "i", "n"],
                                                        dtype="S",
                                                        order="C",
                                                    ),
                                                    [],
                                                    "DataArray_t",
                                                ],
                                            ],
                                            "UserDefinedData_t",
                                        ],
                                    ],
                                    "GridConnectivity_t",
                                ],
                            ],
                            "ZoneGridConnectivity_t",
                        ],
                        [
                            "FlowSolution#EndOfRun",
                            None,
                            [
                                [
                                    "GridLocation",
                                    numpy.array(
                                        [
                                            "C",
                                            "e",
                                            "l",
                                            "l",
                                            "C",
                                            "e",
                                            "n",
                                            "t",
                                            "e",
                                            "r",
                                        ],
                                        dtype="S",
                                        order="C",
                                    ),
                                    [],
                                    "GridLocation_t",
                                ],
                                ["Density", None, [], "DataArray_t"],
                                ["MomentumX", None, [], "DataArray_t"],
                                ["MomentumY", None, [], "DataArray_t"],
                                ["MomentumZ", None, [], "DataArray_t"],
                                ["EnergyStagnationDensity", None, [], "DataArray_t"],
                            ],
                            "FlowSolution_t",
                        ],
                    ],
                    "Zone_t",
                ],
                [
                    "ReferenceState",
                    None,
                    [
                        [
                            "ReferenceStateDescription",
                            numpy.array(
                                [
                                    "G",
                                    "l",
                                    "o",
                                    "b",
                                    "a",
                                    "l",
                                    " ",
                                    "r",
                                    "e",
                                    "f",
                                    "e",
                                    "r",
                                    "e",
                                    "n",
                                    "c",
                                    "e",
                                    " ",
                                    "s",
                                    "t",
                                    "a",
                                    "t",
                                    "e",
                                ],
                                dtype="S",
                                order="C",
                            ),
                            [],
                            "Descriptor_t",
                        ],
                        [
                            "AngleofAttack",
                            numpy.array([7.0], dtype="float64", order="C"),
                            [],
                            "DataArray_t",
                        ],
                        [
                            "BetaAngle",
                            numpy.array([0.0], dtype="float64", order="C"),
                            [],
                            "DataArray_t",
                        ],
                        [
                            "Coef_Area",
                            numpy.array([0.25], dtype="float64", order="C"),
                            [],
                            "DataArray_t",
                        ],
                        [
                            "Coef_PressureDynamic",
                            numpy.array([2837.15315566], dtype="float64", order="C"),
                            [],
                            "DataArray_t",
                        ],
                        [
                            "Density",
                            numpy.array([1.22524863848], dtype="float64", order="C"),
                            [],
                            "DataArray_t",
                        ],
                        [
                            "EnergyStagnationDensity",
                            numpy.array([256154.399197], dtype="float64", order="C"),
                            [],
                            "DataArray_t",
                        ],
                        [
                            "LengthReference",
                            numpy.array([0.5], dtype="float64", order="C"),
                            [],
                            "DataArray_t",
                        ],
                        [
                            "Mach",
                            numpy.array([0.2], dtype="float64", order="C"),
                            [],
                            "DataArray_t",
                        ],
                        [
                            "MomentumX",
                            numpy.array([82.7597580354], dtype="float64", order="C"),
                            [],
                            "DataArray_t",
                        ],
                        [
                            "MomentumY",
                            numpy.array([0.0], dtype="float64", order="C"),
                            [],
                            "DataArray_t",
                        ],
                        [
                            "MomentumZ",
                            numpy.array([10.1616205508], dtype="float64", order="C"),
                            [],
                            "DataArray_t",
                        ],
                        [
                            "Pressure",
                            numpy.array([101326.898416], dtype="float64", order="C"),
                            [],
                            "DataArray_t",
                        ],
                        [
                            "PressureStagnation",
                            numpy.array([104192.536703], dtype="float64", order="C"),
                            [],
                            "DataArray_t",
                        ],
                        [
                            "Reynolds",
                            numpy.array([2330000.0], dtype="float64", order="C"),
                            [],
                            "DataArray_t",
                        ],
                        [
                            "Temperature",
                            numpy.array([288.15], dtype="float64", order="C"),
                            [],
                            "DataArray_t",
                        ],
                        [
                            "TemperatureStagnation",
                            numpy.array([290.4552], dtype="float64", order="C"),
                            [],
                            "DataArray_t",
                        ],
                        [
                            "TurbulentSANuTilde",
                            numpy.array(
                                [1.46035471482e-08], dtype="float64", order="C"
                            ),
                            [],
                            "DataArray_t",
                        ],
                        [
                            "VelocityMagnitude",
                            numpy.array([68.0525297105], dtype="float64", order="C"),
                            [],
                            "DataArray_t",
                        ],
                    ],
                    "ReferenceState_t",
                ],
                [
                    ".Solver#Compute",
                    None,
                    [
                        [
                            "artviscosity",
                            numpy.array(
                                ["d", "i", "s", "s", "c", "a"], dtype="S", order="C"
                            ),
                            [],
                            "DataArray_t",
                        ],
                        [
                            "avcoef_k2",
                            numpy.array([1.0], dtype="float64", order="C"),
                            [],
                            "DataArray_t",
                        ],
                        [
                            "avcoef_k4",
                            numpy.array([0.032], dtype="float64", order="C"),
                            [],
                            "DataArray_t",
                        ],
                        [
                            "avcoef_sigma",
                            numpy.array([1.0], dtype="float64", order="C"),
                            [],
                            "DataArray_t",
                        ],
                        [
                            "cfl",
                            numpy.array([1.0], dtype="float64", order="C"),
                            [],
                            "DataArray_t",
                        ],
                        [
                            "fluid",
                            numpy.array(
                                ["I", "d", "e", "a", "l"], dtype="S", order="C"
                            ),
                            [],
                            "DataArray_t",
                        ],
                        [
                            "flux",
                            numpy.array(
                                ["j", "a", "m", "e", "s", "o", "n"],
                                dtype="S",
                                order="C",
                            ),
                            [],
                            "DataArray_t",
                        ],
                        [
                            "niter",
                            numpy.array([1000], dtype="int32", order="C"),
                            [],
                            "DataArray_t",
                        ],
                        [
                            "ode",
                            numpy.array(["r", "k", "4"], dtype="S", order="C"),
                            [],
                            "DataArray_t",
                        ],
                        [
                            "phymod",
                            numpy.array(
                                ["N", "S", "T", "u", "r", "b", "u", "l", "e", "n", "t"],
                                dtype="S",
                                order="C",
                            ),
                            [],
                            "DataArray_t",
                        ],
                        [
                            "t_cutvar1",
                            numpy.array(
                                [1.78929762604e-08], dtype="float64", order="C"
                            ),
                            [],
                            "DataArray_t",
                        ],
                        [
                            "t_harten",
                            numpy.array([1e-24], dtype="float64", order="C"),
                            [],
                            "DataArray_t",
                        ],
                        [
                            "time_algo",
                            numpy.array(
                                ["s", "t", "e", "a", "d", "y"], dtype="S", order="C"
                            ),
                            [],
                            "DataArray_t",
                        ],
                        [
                            "turbmod",
                            numpy.array([10], dtype="int32", order="C"),
                            [],
                            "DataArray_t",
                        ],
                        [
                            "visclaw",
                            numpy.array(
                                ["S", "u", "t", "h", "e", "r", "l", "a", "n", "d"],
                                dtype="S",
                                order="C",
                            ),
                            [],
                            "DataArray_t",
                        ],
                        [
                            "walldistcompute",
                            numpy.array(
                                [
                                    "m",
                                    "i",
                                    "n",
                                    "i",
                                    "i",
                                    "n",
                                    "t",
                                    "e",
                                    "r",
                                    "f",
                                    "_",
                                    "o",
                                    "r",
                                    "t",
                                    "h",
                                    "o",
                                ],
                                dtype="S",
                                order="C",
                            ),
                            [],
                            "DataArray_t",
                        ],
                    ],
                    "UserDefinedData_t",
                ],
            ],
            "CGNSBase_t",
        ],
    ],
    "CGNSTree_t",
]



#!/usr/bin/env python
# -------------------------------------------------------------------------
# pyCGNS.MAP - CFD General Notation System - SIDS-to-Python MAPping
# See license.txt file in the root directory of this Python module source
# -------------------------------------------------------------------------
#
import CGNS.MAP

print(CGNS.MAP.flags)

for k in CGNS.MAP.flags:
    print(k, CGNS.MAP.__dict__[k])



#  -------------------------------------------------------------------------
#  pyCGNS.NAV - Python package for CFD General Notation System - NAVigater
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
#
import CGNS.PAT.cgnskeywords   as CGK
import CGNS.PAT.cgnsutils      as CGU
import CGNS.APP.lib.arrayutils as CGA

import numpy as NPY

cimport cython
cimport cpython
cimport numpy as CNPY

from qtpy.QtCore import QCoreApplication

import vtk
import vtk.util.numpy_support as vtknpy

vers = vtk.vtkVersion()
VTK_VERSION_MINOR = vers.GetVTKMinorVersion()
VTK_VERSION_MAJOR = vers.GetVTKMajorVersion()

FAMILY_PATTERN = ' :[%s]'

EXTRA_PATTERN = ' :{%s}'
IMIN_PATTERN = EXTRA_PATTERN % 'imin'
JMIN_PATTERN = EXTRA_PATTERN % 'jmin'
KMIN_PATTERN = EXTRA_PATTERN % 'kmin'
IMAX_PATTERN = EXTRA_PATTERN % 'imax'
JMAX_PATTERN = EXTRA_PATTERN % 'jmax'
KMAX_PATTERN = EXTRA_PATTERN % 'kmax'

QUAD_PATTERN = EXTRA_PATTERN % 'quad'
TRI_PATTERN = EXTRA_PATTERN % 'tri '  # mandatory trailing space to have 4 chars

SIZE_PATTERN = len(TRI_PATTERN)
LIST_PATTERN = (IMIN_PATTERN, JMIN_PATTERN, KMIN_PATTERN,
                IMAX_PATTERN, JMAX_PATTERN, KMAX_PATTERN,
                QUAD_PATTERN, TRI_PATTERN)


# ----------------------------------------------------------------------------
class Q7vtkAssembly(vtk.vtkPropAssembly):
    def topo(self, newtopo=None):
        if newtopo is not None:
            self._topo = newtopo
        return self._topo

    def tag(self, newtag=None):
        if newtag is not None:
            self._tag = newtag
        return self._tag

    def alone(self):
        return False

    def __repr__(self):
        s = "Q7vtkAssembly(%s:'%s',%d):<<" % (self._topo, self._tag,
                                              self.GetParts().GetNumberOfItems())
        plst = self.GetParts()
        plst.InitTraversal()
        a = plst.GetNextItemAsObject()
        while a:
            s += a.__repr__()
            a = plst.GetNextItemAsObject()
        s += '>>'
        return s


# ----------------------------------------------------------------------------
class Q7vtkActor(vtk.vtkActor):
    __actortypes = ['BC', 'Min/Max', 'CT']

    def __init__(self, topo=None):
        if (topo in self.__actortypes):
            self._type = topo
        else:
            self._type = '???'  ### FIXME
        self._families = []
        self._zones = []
        self._zonepath = ''

    def topo(self):
        return self._type

    def addFamily(self, fam):
        if fam and fam not in self._families:
            self._families.append(fam)

    def families(self):
        return self._families

    def addZone(self, zone):
        if zone and zone not in self._zones:
            self._zones.append(zone)

    def zones(self):
        return self._zones

    def zonepath(self, zpath=None):
        if zpath is None:
            return self._zonepath
        self._zonepath = zpath

    def path(self, path=None):
        if path is None:
            return self._path
        self._path = path

    def alone(self):
        return True

    def __repr__(self):
        s = " Q7vtkActor('%s':'%s')\n" % (self._type, self._path)
        return s


# ----------------------------------------------------------------------------
class ZoneData(object):
    __zones = []

    def __init__(self, meshcoords, ctlist, bclist, path, ctpaths, bcpaths,
                 sols, bcfams, ctfams):
        self.meshcoordinates = meshcoords  # 0
        self.ctlist = ctlist  # 1
        self.bclist = bclist  # 2
        self.path = path  # 3
        self.ctpaths = ctpaths  # 4
        self.bcpaths = bcpaths  # 5
        self.sols = sols  # 6
        self.bcfams = bcfams
        self.ctfams = ctfams
        self.X = self.meshcoordinates[0]
        self.Y = self.meshcoordinates[1]
        self.Z = self.meshcoordinates[2]
        self.boundaries = zip(self.bcpaths, self.bclist, self.bcfams)
        self.zconnectivities = zip(self.ctpaths, self.ctlist, self.ctfams)
        ZoneData.__zones.append(self)

    @classmethod
    def zonelist(cls):
        for z in cls.__zones:
            yield z


# ----------------------------------------------------------------------------
class CGNSparser(object):
    def __init__(self, tree):
        self._zones = {}
        self._zones_ns = {}
        self._tree = tree
        self._rsd = None

    def resfreshScreen(self):
        QCoreApplication.processEvents()

    def parseZones(self, zlist=[]):
        tree = self._tree
        if tree[0] is None:
            tree[0] = CGK.CGNSTree_s
            tree[3] = CGK.CGNSTree_ts
        for r in CGU.getAllNodesByTypeSet(tree, [CGK.ConvergenceHistory_ts]):
            self._rsd = CGU.nodeByPath(r, tree)
        allfamilies = CGU.getAllFamilies(tree)
        for z in CGU.getAllNodesByTypeSet(tree, [CGK.Zone_ts]):
            zT = CGU.nodeByPath(z, tree)
            if (zlist == []) or (z in zlist):
                gnode = CGU.getAllNodesByTypeSet(zT, [CGK.GridCoordinates_ts])
                if gnode == []:
                    return False
                for g in gnode:
                    zg = g
                    meshpath = zg
                    gT = CGU.nodeByPath(g, zT)
                    cx = CGU.nodeByPath("%s/CoordinateX" % gT[0], gT)
                    cy = CGU.nodeByPath("%s/CoordinateY" % gT[0], gT)
                    cz = CGU.nodeByPath("%s/CoordinateZ" % gT[0], gT)
                    if ((cx is None) or (cy is None) or (cz is None)): return False
                    if ((cx[1] is None) or (cy[1] is None) or (cz[1] is None)):
                        return False
                    zonetype = CGU.getAllNodesByTypeSet(zT, [CGK.ZoneType_ts])
                    ztype = CGU.nodeByPath(zonetype[0], zT)
                    if ztype[1].tostring().decode('ascii') == CGK.Structured_s:
                        if cx[1] is None:
                            return False
                        acx = cx[1]
                        acy = cy[1]
                        acz = cz[1]
                        shx = cx[1].shape
                        scx = cx[1].reshape(shx)
                        scy = cy[1].reshape(shx)
                        scz = cz[1].reshape(shx)
                        simin = [scx[0, :, :], scy[0, :, :], scz[0, :, :]]
                        simax = [scx[-1, :, :], scy[-1, :, :], scz[-1, :, :]]
                        sjmin = [scx[:, 0, :], scy[:, 0, :], scz[:, 0, :]]
                        sjmax = [scx[:, -1, :], scy[:, -1, :], scz[:, -1, :]]
                        skmin = [scx[:, :, 0], scy[:, :, 0], scz[:, :, 0]]
                        skmax = [scx[:, :, -1], scy[:, :, -1], scz[:, :, -1]]
                        zp = zg
                        surfpaths = [zp + IMIN_PATTERN, zp + IMAX_PATTERN,
                                     zp + JMIN_PATTERN, zp + JMAX_PATTERN,
                                     zp + KMIN_PATTERN, zp + KMAX_PATTERN]
                        bclist = []
                        bcpaths = []
                        ctlist = []
                        ctpaths = []
                        bcfams = []
                        ctfams = []
                        solutions = []
                        bname = CGU.getPathAncestor(z)
                        znfams = ['%s/%s' % (bname, znf)
                                  for znf in CGU.getFamiliesFromZone(tree, z)]
                        for sol in CGU.getAllNodesByTypeSet(zT, [CGK.FlowSolution_ts]):
                            zsol = CGU.nodeByPath(sol, zT)
                            for data in CGU.getAllNodesByTypeSet(zsol, [CGK.DataArray_ts]):
                                dsol = CGU.nodeByPath(data, zsol)
                                solutions += [dsol]
                        for nzbc in CGU.getAllNodesByTypeSet(zT, [CGK.ZoneBC_ts]):
                            zbcT = CGU.nodeByPath(nzbc, zT)
                            for nbc in CGU.getAllNodesByTypeSet(zbcT, [CGK.BC_ts]):
                                bcpaths += ['%s/ZoneBC/%s' % (z, nbc.split('/')[1])]
                                bcfams.append(['%s/%s' % (bname, bcf)
                                               for bcf in CGU.getFamiliesFromBC(zT, nbc)])
                                bcfams[-1] += znfams
                                bcT = CGU.nodeByPath(nbc, zbcT)
                                for rbc in CGU.getAllNodesByTypeSet(bcT, [CGK.IndexRange_ts]):
                                    ptr = CGU.nodeByPath(rbc, bcT)[1].T.flat
                                    brg = [scx[ptr[0] - 1:ptr[3], ptr[1] - 1:ptr[4], ptr[2] - 1:ptr[5]],
                                           scy[ptr[0] - 1:ptr[3], ptr[1] - 1:ptr[4], ptr[2] - 1:ptr[5]],
                                           scz[ptr[0] - 1:ptr[3], ptr[1] - 1:ptr[4], ptr[2] - 1:ptr[5]]]
                                    bclist += [brg]
                        for nzbc in CGU.getAllNodesByTypeSet(zT, [CGK.ZoneGridConnectivity_ts]):
                            zbcT = CGU.nodeByPath(nzbc, zT)
                            for nbc in CGU.getAllNodesByTypeSet(zbcT, [CGK.GridConnectivity_ts,
                                                                       CGK.GridConnectivity1to1_ts]):
                                ctpaths += ['%s/ZoneGridConnectivity/%s' % (z, nbc.split('/')[1])]
                                ctfams += [znfams]
                                bcnode = CGU.getNodeByPath(zbcT, nbc)
                                bcT = CGU.nodeByPath(nbc, zbcT)
                                rbc = CGU.hasChildNode(bcT, CGK.PointRange_s)
                                if (rbc is not None):
                                    ptr = rbc[1].T.flat
                                    brg = [scx[ptr[0] - 1:ptr[3], ptr[1] - 1:ptr[4], ptr[2] - 1:ptr[5]],
                                           scy[ptr[0] - 1:ptr[3], ptr[1] - 1:ptr[4], ptr[2] - 1:ptr[5]],
                                           scz[ptr[0] - 1:ptr[3], ptr[1] - 1:ptr[4], ptr[2] - 1:ptr[5]]]
                                    ctlist += [brg]
                        self._zones[zg] = ZoneData([acx, acy, acz],
                                                   ctlist, bclist,
                                                   z, ctpaths, bcpaths, solutions,
                                                   bcfams, ctfams)
                    elif ztype[1].tostring().decode('ascii') == CGK.Unstructured_s:
                        volume = {}
                        surface = {}
                        typeset = [CGK.Elements_ts]
                        elist = CGU.getAllNodesByTypeSet(zT, typeset)
                        sp = CGA.SectionParse()
                        mr = 1
                        sn = 0
                        sl = []
                        for e in elist:
                            sn += 1
                            ne = CGU.getNodeByPath(zT, e)[1]
                            et = ne[0]
                            eb = ne[1]
                            ea = CGU.getNodeByPath(zT, e + '/' + CGK.ElementConnectivity_s)[1]
                            if (ea is not None) and (et in sp.QUAD_SURFACE):
                                pth = CGU.getPathAncestor(meshpath) + e + QUAD_PATTERN
                                sl.append(list(sp.extQuadFacesPoints(ea, et, sn, mr, eb)) + [pth])
                            if ((ea is not None) and (et in sp.TRI_SURFACE)):
                                pth = e + TRI_PATTERN
                                sl.append(list(sp.extTriFacesPoints(ea, et, sn, mr, eb)) + [pth])
                        self._zones_ns[z] = ([cx[1], cy[1], cz[1]], meshpath, et, sl)
        return True


# ----------------------------------------------------------------------------
class Mesh(CGNSparser):
    def __init__(self, tree, zlist):
        CGNSparser.__init__(self, tree)
        self._color = (1, 0, 0)
        self._actors = []
        self._vtkelts = {CGK.TRI_3: (vtk.vtkTriangle, (3, 3)),
                         CGK.TRI_6: (vtk.vtkTriangle, (3, 6)),
                         CGK.QUAD_4: (vtk.vtkQuad, (4, 4)),
                         CGK.QUAD_8: (vtk.vtkQuad, (4, 8)),
                         CGK.QUAD_9: (vtk.vtkQuad, (4, 9)),
                         CGK.TETRA_4: (vtk.vtkTetra, (4, 4)),
                         CGK.TETRA_10: (vtk.vtkTetra, (4, 10)),
                         CGK.PYRA_5: (vtk.vtkPyramid, (5, 5)),
                         CGK.PYRA_14: (vtk.vtkPyramid, (5, 14)),
                         CGK.HEXA_8: (vtk.vtkHexahedron, (8, 8)),
                         CGK.HEXA_20: (vtk.vtkHexahedron, (8, 20)),
                         CGK.HEXA_27: (vtk.vtkHexahedron, (8, 27))}

        if VTK_VERSION_MAJOR <  8:
            raise RuntimeError("VTK version is too old, please upgrade.")

        self._vtkelts[CGK.PENTA_6] = (vtk.vtkPolyhedron, (6, 6))
        self._vtkelts[CGK.PENTA_15] = (vtk.vtkPolyhedron, (6, 15))
        self._vtkelts[CGK.PENTA_18] = (vtk.vtkPolyhedron, (6, 18))

        try:
            self._status = self.parseZones(zlist)
        except (ValueError, IndexError):
            self._status = False

    def getResidus(self):
        return self._rsd

    def createActors(self):
        for z in ZoneData.zonelist():
            self.do_vtk(z)
            QCoreApplication.processEvents()
        self._actors += self.createActors_ns()
        lf = self.addFamilies()
        self._actors += lf
        lf = self.addIntoZones()
        self._actors += lf
        QCoreApplication.processEvents()
        return self._actors

    def addFamilies(self):
        fd = {}
        for a in self._actors:
            for f in a[0].families():
                if (f and f not in fd):
                    g = a[0].GetMapper().GetInput()
                    fd[f] = [Q7vtkAssembly(), None, g, f, (1, None), None]
                fd[f][0].AddPart(a[0])
                fd[f][0].topo('Family')
                fd[f][0].tag(f)
        return fd.values()

    def addIntoZones(self):
        fd = {}
        for a in self._actors:
            if (a[0].alone()):
                for zn in ZoneData.zonelist():
                    if (a[0].zonepath() == zn.path):
                        if (zn.path not in fd):
                            g = a[0].GetMapper().GetInput()
                            fd[zn.path] = [Q7vtkAssembly(), None, g, zn.path, (1, None), None]
                        fd[zn.path][0].AddPart(a[0])
                        fd[zn.path][0].topo('Zone')
                        fd[zn.path][0].tag(zn.path)
        return fd.values()

    def createActors_ns(self):
        actors = self.do_surface_ns(self._zones_ns)
        return actors

    def getObjectList(self):
        return self._actors
        # self._actor := [ vtk.vtkActor, bbox, vtk.vtk<grid>, path, dims ]

    def getPathList(self, filter=[]):
        if filter == []:
            r = [a[3] for a in self._actors]
            r.sort()
            return r
        else:
            r = []
            for a in self._actors:
                if a[0].topo() in filter:
                    r.append(a[3])
            r.sort()
            return r

    def getFamilyList(self):
        return ('A', 'B', 'C')

    def getPathFromObject(self, selectedobject):
        for (o, p) in [(a[2], a[3]) for a in self._actors]:
            print('get path :', p, selectedobject, o)
            if selectedobject == o:
                return p
        return ''

    def getObjectFromPath(self, selectedpath):
        for (o, p) in [(a[2], a[3]) for a in self._actors]:
            if (selectedpath == p): return o
        return ''

    def getDimsFromObject(self, selectedobject):
        for (o, p) in [(a[2], a[4]) for a in self._actors]:
            if selectedobject == o:
                return p
        return (1, None)

    #  @cython.boundscheck(False)
    def do_volume(self, path, CNPY.ndarray dx, CNPY.ndarray dy, CNPY.ndarray dz, solution):
        data = vtk.vtkIntArray()
        data.SetNumberOfComponents(3)
        data.SetName("index volume")
        cdef int p, i, j, k
        cdef int ijkDim[3]
        cdef float xf, yf, zf
        cdef double xd, yd, zd
        ijkDim[0] = 1
        ijkDim[1] = 1
        ijkDim[2] = 1
        for idx in range(dx.ndim):
           ijkDim[idx] = dx.shape[idx]
        pts = vtk.vtkPoints()
        pts.SetNumberOfPoints(ijkDim[0] * ijkDim[1] * ijkDim[2])
        if (dx.dtype == NPY.float32):
            for k in range(ijkDim[2]):
                for j in range(ijkDim[1]):
                    for i in range(ijkDim[0]):
                        data.InsertNextTuple3(i + 1, j + 1, k + 1)
                        p = i + j * ijkDim[0] + k * ijkDim[0] * ijkDim[1]
                        xf = ( < float * > CNPY.PyArray_GETPTR1(dx, p))[0]
                        yf = ( < float * > CNPY.PyArray_GETPTR1(dy, p))[0]
                        zf = ( < float * > CNPY.PyArray_GETPTR1(dz, p))[0]
                        pts.InsertPoint(p, xf, yf, zf)
        else:
            for k in range(ijkDim[2]):
                for j in range(ijkDim[1]):
                    for i in range(ijkDim[0]):
                        data.InsertNextTuple3(i + 1, j + 1, k + 1)
                        p = i + j * ijkDim[0] + k * ijkDim[0] * ijkDim[1]
                        xd = (( < double * > dx.data) + p)[0]
                        yd = (( < double * > dy.data) + p)[0]
                        zd = (( < double * > dz.data) + p)[0]
                        pts.InsertPoint(p, xd, yd, zd)
        g = vtk.vtkStructuredGrid()
        g.SetPoints(pts)
        g.SetExtent(0, ijkDim[0] - 1, 0, ijkDim[1] - 1, 0, ijkDim[2] - 1)
        d = vtk.vtkDataSetMapper()
        if VTK_VERSION_MAJOR < 8:
           raise RuntimeError("VTK version is too old, please upgrade.")
        d.SetInputData(g)
        a = Q7vtkActor('Zone')
        a.SetMapper(d)
        a.GetProperty().SetRepresentationToWireframe()
        g.GetPointData().AddArray(data)
        for s in solution:
            if ((s[1] is not None) and (s[1].shape == (ijkDim[0] - 1, ijkDim[1] - 1, ijkDim[2]- 1))):
                array = vtk.vtkFloatArray()
                array.SetName(s[0])
                for k in range(ijkDim[2] - 1):
                    for j in range(ijkDim[1] - 1):
                        for i in range(ijkDim[0] - 1):
                            array.InsertNextTuple1(s[1][i][j][k])
                g.GetCellData().AddArray(array)
        return (a, a.GetBounds(), g, path, (0, (ijkDim[0], ijkDim[1], ijkDim[2])))

    #  @cython.boundscheck(False)
    def do_surface_double_3d(self, path, surf):
        cdef int n, np, i, j, imax, jmax, p1, p2, p3, p4
        cdef double xs, ys, zs
        cdef CNPY.ndarray[CNPY.float64_t, ndim = 2] _tx
        cdef CNPY.ndarray[CNPY.float64_t, ndim = 2] _ty
        cdef CNPY.ndarray[CNPY.float64_t, ndim = 2] _tz
        cdef double * tx
        cdef double * ty
        cdef double * tz
        imax = surf[0].shape[0]
        jmax = surf[0].shape[1]
        _tx = surf[0]
        _ty = surf[1]
        _tz = surf[2]
        tx = < double * > _tx.data
        ty = < double * > _ty.data
        tz = < double * > _tz.data
        sg = vtk.vtkUnstructuredGrid()
        sg.Allocate(1, 1)
        n = 0
        qp = vtk.vtkPoints()  # TODO: add family label as attribute
        for j in range(jmax - 1):
            for i in range(imax - 1):
                p1 = j + i * jmax + 0
                p2 = j + i * jmax + 1
                p3 = j + jmax + i * jmax + 1
                p4 = j + jmax + i * jmax + 0
                aq = vtk.vtkQuad()
                aqp = aq.GetPointIds()
                np = n * 4
                xs = tx[p1]
                ys = ty[p1]
                zs = tz[p1]
                qp.InsertPoint(np, xs, ys, zs)
                aqp.SetId(0, np)
                np += 1
                qp.InsertPoint(np, tx[p2], ty[p2], tz[p2])
                aqp.SetId(1, np)
                np += 1
                qp.InsertPoint(np, tx[p3], ty[p3], tz[p3])
                aqp.SetId(2, np)
                np += 1
                qp.InsertPoint(np, tx[p4], ty[p4], tz[p4])
                aqp.SetId(3, np)
                sg.InsertNextCell(aq.GetCellType(), aqp)
                n += 1
        qp = vtk.vtkPoints()
        sg.SetPoints(qp)
        am = vtk.vtkDataSetMapper()
        if VTK_VERSION_MAJOR < 8:
            raise RuntimeError("VTK version is too old, please upgrade.")
        am.SetInputData(sg)
        a = Q7vtkActor('Min/Max')
        a.SetMapper(am)
        a.GetProperty().SetRepresentationToWireframe()
        return (a, None, sg, path, (1, None), None)

    def do_boundaries(self, path, bnd, fams, tag='BC', zone=''):
        cdef int i, j, imax, jmax, p1, p2, p3, p4
        max = [x for x in bnd[0].shape if x != 1]
        imax = max[0]
        jmax = max[1]
        tx = bnd[0].flat
        ty = bnd[1].flat
        tz = bnd[2].flat
        sg = vtk.vtkUnstructuredGrid()
        sg.Allocate(1, 1)
        n = 0
        qp = vtk.vtkPoints()
        for j in range(jmax - 1):
            for i in range(imax - 1):
                p1 = j + i * jmax + 0
                p2 = j + i * jmax + 1
                p3 = j + jmax + i * jmax + 1
                p4 = j + jmax + i * jmax + 0
                qp.InsertPoint(n * 4 + 0, tx[p1], ty[p1], tz[p1])
                qp.InsertPoint(n * 4 + 1, tx[p2], ty[p2], tz[p2])
                qp.InsertPoint(n * 4 + 2, tx[p3], ty[p3], tz[p3])
                qp.InsertPoint(n * 4 + 3, tx[p4], ty[p4], tz[p4])
                aq = vtk.vtkQuad()
                aq.GetPointIds().SetId(0, n * 4 + 0)
                aq.GetPointIds().SetId(1, n * 4 + 1)
                aq.GetPointIds().SetId(2, n * 4 + 2)
                aq.GetPointIds().SetId(3, n * 4 + 3)
                sg.InsertNextCell(aq.GetCellType(), aq.GetPointIds())
                n += 1
        sg.SetPoints(qp)
        am = vtk.vtkDataSetMapper()
        if VTK_VERSION_MAJOR < 8:
            raise RuntimeError("VTK version is too old, please upgrade.")
        am.SetInputData(sg)
        a = Q7vtkActor(tag)
        a.SetMapper(am)
        a.GetProperty().SetRepresentationToWireframe()
        for f in fams:
            a.addFamily(f)
        a.path(path)
        a.zonepath(zone)
        return (a, None, sg, path, (1, None), None)

    def do_vtk(self, z):
        # self._actors+=[self.do_volume(z.path,z.X,z.Y,z.Z,z.sols)]
        for (path, coords, fams) in z.zconnectivities:
            self._actors += [self.do_boundaries(path, coords, fams, 'CT', zone=z.path)]
        for (path, coords, fams) in z.boundaries:
            self._actors += [self.do_boundaries(path, coords, fams, zone=z.path)]
        return

    #  @cython.boundscheck(False)
    def do_surface_ns(self, zones):
        cdef int e, elts, n, npe, idg, idx
        actors = []
        for zn in zones:
            dx = zones[zn][0][0]
            dy = zones[zn][0][1]
            dz = zones[zn][0][2]
            for surf in zones[zn][-1]:
                path = surf[-1]
                if (surf[0] == CGK.QUAD_4):
                    npe = 4
                else:
                    npe = 3
                sf = surf[1]
                elts = len(sf) // npe
                sg = vtk.vtkUnstructuredGrid()
                sg.Allocate(1, 1)
                e = 0
                qp = vtk.vtkPoints()
                vtkelt = self.def_volume(surf[0])[0]
                while e < elts:
                    n = 0
                    aq = vtkelt()
                    while n < npe:
                        idg = (e * npe + n)
                        ids = sf[idg] - 1
                        qp.InsertPoint(idg, dx[ids], dy[ids], dz[ids])
                        aq.GetPointIds().SetId(n, idg)
                        n += 1
                    sg.InsertNextCell(aq.GetCellType(), aq.GetPointIds())
                    e += 1
                sg.SetPoints(qp)
                am = vtk.vtkDataSetMapper()
                if VTK_VERSION_MAJOR < 8:
                    raise RuntimeError("VTK version is too old, please upgrade.")
                am.SetInputData(sg)
                a = Q7vtkActor()
                a.SetMapper(am)
                a.GetProperty().SetRepresentationToWireframe()
                actors += [(a, a.GetBounds(), sg, path, (2, None))]
        return actors

    def def_volume(self, n):
        return self._vtkelts.get(n, None)

# ----------------------------------------------------------------------------



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'CGNS/NAV/T/Q7AnimationWindow.ui'
#
# Created by: PyQt5 UI code generator 5.12.3
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Q7AnimationWindow(object):
    def setupUi(self, Q7AnimationWindow):
        Q7AnimationWindow.setObjectName("Q7AnimationWindow")
        Q7AnimationWindow.resize(715, 350)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(Q7AnimationWindow.sizePolicy().hasHeightForWidth())
        Q7AnimationWindow.setSizePolicy(sizePolicy)
        Q7AnimationWindow.setMinimumSize(QtCore.QSize(715, 350))
        Q7AnimationWindow.setMaximumSize(QtCore.QSize(1200, 350))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/images/icons/cgSpy.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        Q7AnimationWindow.setWindowIcon(icon)
        self.gridLayout = QtWidgets.QGridLayout(Q7AnimationWindow)
        self.gridLayout.setObjectName("gridLayout")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.bBackControl = QtWidgets.QPushButton(Q7AnimationWindow)
        self.bBackControl.setMinimumSize(QtCore.QSize(25, 25))
        self.bBackControl.setMaximumSize(QtCore.QSize(25, 25))
        self.bBackControl.setText("")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/images/icons/top.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bBackControl.setIcon(icon1)
        self.bBackControl.setObjectName("bBackControl")
        self.horizontalLayout_2.addWidget(self.bBackControl)
        self.bInfo = QtWidgets.QPushButton(Q7AnimationWindow)
        self.bInfo.setMinimumSize(QtCore.QSize(25, 25))
        self.bInfo.setMaximumSize(QtCore.QSize(25, 25))
        self.bInfo.setText("")
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/images/icons/help-view.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bInfo.setIcon(icon2)
        self.bInfo.setObjectName("bInfo")
        self.horizontalLayout_2.addWidget(self.bInfo)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.bClose = QtWidgets.QPushButton(Q7AnimationWindow)
        self.bClose.setObjectName("bClose")
        self.horizontalLayout_2.addWidget(self.bClose)
        self.gridLayout.addLayout(self.horizontalLayout_2, 6, 0, 1, 1)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.diagTable = QtWidgets.QTreeWidget(Q7AnimationWindow)
        self.diagTable.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOn)
        self.diagTable.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOn)
        self.diagTable.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.diagTable.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.diagTable.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.diagTable.setObjectName("diagTable")
        self.diagTable.headerItem().setText(0, "1")
        self.verticalLayout.addWidget(self.diagTable)
        self.gridLayout.addLayout(self.verticalLayout, 5, 0, 1, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.bExpandAll = QtWidgets.QPushButton(Q7AnimationWindow)
        self.bExpandAll.setMinimumSize(QtCore.QSize(25, 25))
        self.bExpandAll.setMaximumSize(QtCore.QSize(25, 25))
        self.bExpandAll.setText("")
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/images/icons/level-in.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bExpandAll.setIcon(icon3)
        self.bExpandAll.setObjectName("bExpandAll")
        self.horizontalLayout.addWidget(self.bExpandAll)
        self.bCollapseAll = QtWidgets.QPushButton(Q7AnimationWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(25)
        sizePolicy.setVerticalStretch(25)
        sizePolicy.setHeightForWidth(self.bCollapseAll.sizePolicy().hasHeightForWidth())
        self.bCollapseAll.setSizePolicy(sizePolicy)
        self.bCollapseAll.setMinimumSize(QtCore.QSize(25, 25))
        self.bCollapseAll.setMaximumSize(QtCore.QSize(25, 25))
        self.bCollapseAll.setText("")
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/images/icons/level-out.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bCollapseAll.setIcon(icon4)
        self.bCollapseAll.setObjectName("bCollapseAll")
        self.horizontalLayout.addWidget(self.bCollapseAll)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.bFrameAdd = QtWidgets.QPushButton(Q7AnimationWindow)
        self.bFrameAdd.setMinimumSize(QtCore.QSize(25, 25))
        self.bFrameAdd.setMaximumSize(QtCore.QSize(25, 25))
        self.bFrameAdd.setText("")
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/images/icons/anim-add.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bFrameAdd.setIcon(icon5)
        self.bFrameAdd.setObjectName("bFrameAdd")
        self.horizontalLayout.addWidget(self.bFrameAdd)
        self.bFrameDel = QtWidgets.QPushButton(Q7AnimationWindow)
        self.bFrameDel.setMinimumSize(QtCore.QSize(25, 25))
        self.bFrameDel.setMaximumSize(QtCore.QSize(25, 25))
        self.bFrameDel.setText("")
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(":/images/icons/anim-del.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bFrameDel.setIcon(icon6)
        self.bFrameDel.setObjectName("bFrameDel")
        self.horizontalLayout.addWidget(self.bFrameDel)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem2)
        self.pushButton_7 = QtWidgets.QPushButton(Q7AnimationWindow)
        self.pushButton_7.setMinimumSize(QtCore.QSize(25, 25))
        self.pushButton_7.setMaximumSize(QtCore.QSize(25, 25))
        self.pushButton_7.setText("")
        icon7 = QtGui.QIcon()
        icon7.addPixmap(QtGui.QPixmap(":/images/icons/anim-auto-sids.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_7.setIcon(icon7)
        self.pushButton_7.setObjectName("pushButton_7")
        self.horizontalLayout.addWidget(self.pushButton_7)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem3)
        self.bSave = QtWidgets.QPushButton(Q7AnimationWindow)
        self.bSave.setMinimumSize(QtCore.QSize(25, 25))
        self.bSave.setMaximumSize(QtCore.QSize(25, 25))
        self.bSave.setText("")
        icon8 = QtGui.QIcon()
        icon8.addPixmap(QtGui.QPixmap(":/images/icons/anim-save.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bSave.setIcon(icon8)
        self.bSave.setObjectName("bSave")
        self.horizontalLayout.addWidget(self.bSave)
        self.gridLayout.addLayout(self.horizontalLayout, 3, 0, 1, 1)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.pushButton = QtWidgets.QPushButton(Q7AnimationWindow)
        self.pushButton.setMinimumSize(QtCore.QSize(25, 25))
        self.pushButton.setMaximumSize(QtCore.QSize(25, 25))
        self.pushButton.setText("")
        icon9 = QtGui.QIcon()
        icon9.addPixmap(QtGui.QPixmap(":/images/icons/anim-item-first.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton.setIcon(icon9)
        self.pushButton.setObjectName("pushButton")
        self.horizontalLayout_3.addWidget(self.pushButton)
        self.pushButton_2 = QtWidgets.QPushButton(Q7AnimationWindow)
        self.pushButton_2.setMinimumSize(QtCore.QSize(25, 25))
        self.pushButton_2.setMaximumSize(QtCore.QSize(25, 25))
        self.pushButton_2.setText("")
        icon10 = QtGui.QIcon()
        icon10.addPixmap(QtGui.QPixmap(":/images/icons/anim-item-prev.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_2.setIcon(icon10)
        self.pushButton_2.setObjectName("pushButton_2")
        self.horizontalLayout_3.addWidget(self.pushButton_2)
        self.pushButton_5 = QtWidgets.QPushButton(Q7AnimationWindow)
        self.pushButton_5.setMinimumSize(QtCore.QSize(25, 25))
        self.pushButton_5.setMaximumSize(QtCore.QSize(25, 25))
        self.pushButton_5.setText("")
        icon11 = QtGui.QIcon()
        icon11.addPixmap(QtGui.QPixmap(":/images/icons/anim-item-next.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_5.setIcon(icon11)
        self.pushButton_5.setObjectName("pushButton_5")
        self.horizontalLayout_3.addWidget(self.pushButton_5)
        self.pushButton_6 = QtWidgets.QPushButton(Q7AnimationWindow)
        self.pushButton_6.setMinimumSize(QtCore.QSize(25, 25))
        self.pushButton_6.setMaximumSize(QtCore.QSize(25, 25))
        self.pushButton_6.setText("")
        icon12 = QtGui.QIcon()
        icon12.addPixmap(QtGui.QPixmap(":/images/icons/anim-item-last.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_6.setIcon(icon12)
        self.pushButton_6.setObjectName("pushButton_6")
        self.horizontalLayout_3.addWidget(self.pushButton_6)
        self.cActors = QtWidgets.QComboBox(Q7AnimationWindow)
        self.cActors.setObjectName("cActors")
        self.horizontalLayout_3.addWidget(self.cActors)
        self.pushButton_3 = QtWidgets.QPushButton(Q7AnimationWindow)
        self.pushButton_3.setMinimumSize(QtCore.QSize(25, 25))
        self.pushButton_3.setMaximumSize(QtCore.QSize(25, 25))
        self.pushButton_3.setText("")
        icon13 = QtGui.QIcon()
        icon13.addPixmap(QtGui.QPixmap(":/images/icons/anim-item-add.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_3.setIcon(icon13)
        self.pushButton_3.setObjectName("pushButton_3")
        self.horizontalLayout_3.addWidget(self.pushButton_3)
        self.pushButton_4 = QtWidgets.QPushButton(Q7AnimationWindow)
        self.pushButton_4.setMinimumSize(QtCore.QSize(25, 25))
        self.pushButton_4.setMaximumSize(QtCore.QSize(25, 25))
        self.pushButton_4.setText("")
        icon14 = QtGui.QIcon()
        icon14.addPixmap(QtGui.QPixmap(":/images/icons/anim-item-del.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_4.setIcon(icon14)
        self.pushButton_4.setObjectName("pushButton_4")
        self.horizontalLayout_3.addWidget(self.pushButton_4)
        self.gridLayout.addLayout(self.horizontalLayout_3, 4, 0, 1, 1)

        self.retranslateUi(Q7AnimationWindow)
        QtCore.QMetaObject.connectSlotsByName(Q7AnimationWindow)

    def retranslateUi(self, Q7AnimationWindow):
        _translate = QtCore.QCoreApplication.translate
        Q7AnimationWindow.setWindowTitle(_translate("Q7AnimationWindow", "Form"))
        self.bClose.setText(_translate("Q7AnimationWindow", "Close"))
from . import Res_rc
